@@ -1,9 +1,7 @@
 from draive.anthropic.client import Anthropic
 from draive.anthropic.config import AnthropicConfig
-from draive.anthropic.types import AnthropicException
 
 __all__ = (
     "Anthropic",
     "AnthropicConfig",
-    "AnthropicException",
 )
