@@ -1,8 +1,6 @@
 from collections.abc import Sequence
 
-from haiway import MISSING, Missing
-
-from draive.configuration import Config
+from haiway import MISSING, Configuration, Missing
 
 __all__ = (
     "BedrockChatConfig",
@@ -11,20 +9,19 @@ __all__ = (
 )
 
 
-class BedrockChatConfig(Config):
+class BedrockChatConfig(Configuration):
     model: str
     temperature: float = 1.0
     top_p: float | Missing = MISSING
-    max_tokens: int | Missing = MISSING
-    timeout: float | Missing = MISSING
+    max_output_tokens: int | Missing = MISSING
     stop_sequences: Sequence[str] | Missing = MISSING
 
 
-class BedrockInputGuardraisConfig(Config):
+class BedrockInputGuardraisConfig(Configuration):
     guardrail_identifier: str
     guardrail_version: str
 
 
-class BedrockOutputGuardraisConfig(Config):
+class BedrockOutputGuardraisConfig(Configuration):
     guardrail_identifier: str
     guardrail_version: str
