@@ -1,3 +1,4 @@
+from base64 import urlsafe_b64decode
 from typing import Any, Literal
 
 from haiway import ObservabilityLevel, asynchronous, ctx
@@ -5,13 +6,14 @@ from haiway import ObservabilityLevel, asynchronous, ctx
 from draive.bedrock.api import BedrockAPI
 from draive.bedrock.config import BedrockInputGuardraisConfig, BedrockOutputGuardraisConfig
 from draive.guardrails import GuardrailsModeration, GuardrailsModerationException
-from draive.multimodal import MediaData, Multimodal, MultimodalContent, TextContent
+from draive.multimodal import Multimodal, MultimodalContent, TextContent
+from draive.resources import ResourceContent, ResourceReference
 
-__all__ = ("BedrockGuardrais",)
+__all__ = ("BedrockGuardrails",)
 
 
-class BedrockGuardrais(BedrockAPI):
-    def content_guardrails(self) -> GuardrailsModeration:
+class BedrockGuardrails(BedrockAPI):
+    def guardrails_moderation(self) -> GuardrailsModeration:
         return GuardrailsModeration(
             input_checking=self.content_input_verification,
             output_checking=self.content_output_verification,
@@ -27,7 +29,7 @@ class BedrockGuardrais(BedrockAPI):
     ) -> None:
         guardrails_config: BedrockInputGuardraisConfig = config or ctx.state(
             BedrockInputGuardraisConfig
-        ).updated(**extra)
+        )
 
         content = MultimodalContent.of(content)
         await self._content_input_verification(
@@ -57,12 +59,12 @@ class BedrockGuardrais(BedrockAPI):
         if response.get("action") == "GUARDRAIL_INTERVENED":
             raise GuardrailsModerationException(
                 f"Content violated guardrails ({config.guardrail_identifier}) rules",
-                violations=tuple(
-                    filter["type"]
+                violations={
+                    filter["type"]: 1.0  # bedrock do not provide scores in response
                     for assessment in response.get("assessments", ())
                     for filter in assessment.get("contentPolicy", {}).get("filters", ())  # noqa: A001
                     if "type" in filter and filter.get("detected", False)
-                ),
+                },
                 content=content,
             )
 
@@ -77,7 +79,7 @@ class BedrockGuardrais(BedrockAPI):
         guardrails_config: BedrockOutputGuardraisConfig = config or ctx.state(
             BedrockOutputGuardraisConfig
         )
-        async with ctx.scope("bedrock_guardrails", guardrails_config):
+        async with ctx.scope("bedrock_guardrails"):
             ctx.record(
                 ObservabilityLevel.INFO,
                 attributes={
@@ -116,7 +118,8 @@ class BedrockGuardrais(BedrockAPI):
         if response.get("action") == "GUARDRAIL_INTERVENED":
             raise GuardrailsModerationException(
                 f"Violated guardrails rules: {response.get('actionReason')}",
-                violations=(config.guardrail_identifier,),
+                # bedrock do not provide scores in response
+                violations={config.guardrail_identifier: 1.0},
                 content=content,
             )
 
@@ -140,51 +143,75 @@ class BedrockGuardrais(BedrockAPI):
                         }
                     )
 
-                case MediaData() as media_data:
-                    # Check media type properly using startswith for MIME types
-                    if media_data.media.startswith("image/jpeg"):
+                case ResourceContent() as content_part:
+                    # Only selected image resources are supported by Bedrock guardrails
+                    if content_part.mime_type == "image/jpeg":
                         moderated_content.append(
                             {
                                 "image": {
                                     "format": "jpeg",
-                                    "source": {"bytes": media_data.data},
+                                    "source": {"bytes": urlsafe_b64decode(content_part.data)},
                                 }
                             }
                         )
 
-                    elif media_data.media.startswith("image/png"):
+                    elif content_part.mime_type == "image/png":
                         moderated_content.append(
                             {
                                 "image": {
                                     "format": "png",
-                                    "source": {"bytes": media_data.data},
+                                    "source": {"bytes": urlsafe_b64decode(content_part.data)},
+                                }
+                            }
+                        )
+
+                    elif content_part.mime_type == "image/gif":
+                        moderated_content.append(
+                            {
+                                "image": {
+                                    "format": "gif",
+                                    "source": {"bytes": urlsafe_b64decode(content_part.data)},
                                 }
                             }
                         )
 
                     else:
                         ctx.log_warning(
-                            f"Attempting to use guardrails on unsupported media"
-                            f" ({media_data.media}), verifying as json text..."
+                            "Attempting to use guardrails on unsupported media "
+                            f"({content_part.mime_type}), verifying as text..."
                         )
                         moderated_content.append(
                             {
                                 "text": {
-                                    "text": media_data.to_json(),
+                                    "text": content_part.to_str(),
                                     "qualifiers": [qualifier],
                                 }
                             }
                         )
 
-                case other:
+                case ResourceReference() as ref:
                     ctx.log_warning(
-                        f"Attempting to use guardrails on unsupported content ({type(other)}),"
-                        " verifying as json text..."
+                        "Bedrock guardrails require image bytes; got ResourceReference. "
+                        "Verifying reference as JSON text."
                     )
                     moderated_content.append(
                         {
                             "text": {
-                                "text": other.to_json(),
+                                "text": ref.to_str(),
+                                "qualifiers": [qualifier],
+                            }
+                        }
+                    )
+
+                case other:
+                    ctx.log_warning(
+                        f"Attempting to use guardrails on unsupported content ({type(other)}),"
+                        " verifying as text..."
+                    )
+                    moderated_content.append(
+                        {
+                            "text": {
+                                "text": other.to_str(),
                                 "qualifiers": [qualifier],
                             }
                         }
