@@ -1,10 +1,8 @@
 from draive.cohere.client import Cohere
 from draive.cohere.config import CohereImageEmbeddingConfig, CohereTextEmbeddingConfig
-from draive.cohere.types import CohereException
 
 __all__ = (
     "Cohere",
-    "CohereException",
     "CohereImageEmbeddingConfig",
     "CohereTextEmbeddingConfig",
 )
