@@ -1,27 +1,31 @@
-from collections.abc import AsyncIterator
+from collections.abc import AsyncGenerator
 from typing import Any, Literal, Protocol, overload, runtime_checkable
 
 from draive.conversation.types import (
-    ConversationMemory,
     ConversationMessage,
-    ConversationStreamElement,
+    ConversationOutputChunk,
 )
-from draive.instructions import Instruction
-from draive.tools import Toolbox
+from draive.models import ModelMemory, ResolveableInstructions, Toolbox
 
 __all__ = ("ConversationCompleting",)
 
 
 @runtime_checkable
 class ConversationCompleting(Protocol):
+    """Provider interface for a single-turn conversation completion.
+
+    Implementations may produce a full response message or stream output chunks
+    depending on the ``stream`` flag.
+    """
+
     @overload
     async def __call__(
         self,
         *,
-        instruction: Instruction | None,
-        input: ConversationMessage,
-        memory: ConversationMemory,
+        instructions: ResolveableInstructions,
         toolbox: Toolbox,
+        memory: ModelMemory,
+        input: ConversationMessage,
         stream: Literal[False] = False,
         **extra: Any,
     ) -> ConversationMessage: ...
@@ -30,21 +34,21 @@ class ConversationCompleting(Protocol):
     async def __call__(
         self,
         *,
-        instruction: Instruction | None,
-        input: ConversationMessage,
-        memory: ConversationMemory,
+        instructions: ResolveableInstructions,
         toolbox: Toolbox,
+        memory: ModelMemory,
+        input: ConversationMessage,
         stream: Literal[True],
         **extra: Any,
-    ) -> AsyncIterator[ConversationStreamElement]: ...
+    ) -> AsyncGenerator[ConversationOutputChunk]: ...
 
     async def __call__(
         self,
         *,
-        instruction: Instruction | None,
-        input: ConversationMessage,  # noqa: A002
-        memory: ConversationMemory,
+        instructions: ResolveableInstructions,
         toolbox: Toolbox,
+        memory: ModelMemory,
+        input: ConversationMessage,  # noqa: A002
         stream: bool = False,
         **extra: Any,
-    ) -> AsyncIterator[ConversationStreamElement] | ConversationMessage: ...
+    ) -> AsyncGenerator[ConversationOutputChunk] | ConversationMessage: ...
