@@ -1,8 +1,6 @@
 from asyncio import CancelledError
-from collections.abc import Sequence
 from types import TracebackType
 from typing import Any
-from uuid import uuid4
 
 from haiway import ctx
 
@@ -12,69 +10,63 @@ from draive.conversation.realtime.types import (
 )
 from draive.conversation.types import (
     ConversationEvent,
-    ConversationMemory,
-    ConversationMessage,
-    ConversationMessageChunk,
-    ConversationStreamElement,
+    ConversationInputChunk,
+    ConversationOutputChunk,
 )
-from draive.instructions import Instruction
-from draive.lmm import (
-    LMMContext,
-    LMMContextElement,
-    LMMMemory,
-    LMMSession,
-    LMMSessionEvent,
-    LMMSessionScope,
-    LMMStreamChunk,
-    LMMToolRequest,
-    LMMToolResponse,
-    RealtimeLMM,
+from draive.models import (
+    ModelMemory,
+    ModelSession,
+    ModelSessionOutputSelection,
+    ModelSessionScope,
+    RealtimeGenerativeModel,
+    ResolveableInstructions,
+    Toolbox,
+)
+from draive.models.types import (
+    ModelInputChunk,
+    ModelOutputChunk,
+    ModelSessionEvent,
+    ModelToolRequest,
+    ModelToolResponse,
 )
-from draive.lmm.types import LMMCompletion, LMMInput
-from draive.tools import Toolbox
-from draive.utils import MEMORY_NONE, Memory, MemoryRecalling, MemoryRemembering
 
 __all__ = ("realtime_conversation_preparing",)
 
 
 async def realtime_conversation_preparing(  # noqa: C901
     *,
-    instruction: Instruction | None,
-    memory: ConversationMemory | None,
+    instructions: ResolveableInstructions,
     toolbox: Toolbox,
+    memory: ModelMemory,
+    output: ModelSessionOutputSelection,
     **extra: Any,
 ) -> RealtimeConversationSessionScope:
-    session_memory: LMMMemory
-    match memory:
-        case None:
-            session_memory = MEMORY_NONE
-
-        case Memory():
-            session_memory = Memory(
-                recall=_to_lmm_recall(memory.recall),
-                remember=_to_lmm_remember(memory.remember),
-            )
+    """Default realtime conversation session preparation.
 
+    Opens a realtime generative model session, adapts session I/O to conversation-level
+    chunks, and wires tool requests handling using the provided toolbox.
+    """
     # TODO: rework RealtimeLMM interface to allow instruction and tools changes during the session
-    session_scope: LMMSessionScope = await RealtimeLMM.session(
-        instruction=Instruction.formatted(instruction),
-        memory=session_memory,
-        tools=toolbox.available_tools(),
-        output="auto",
+    session_scope: ModelSessionScope = await RealtimeGenerativeModel.session(
+        instructions=instructions,
+        memory=memory,
+        tools=toolbox.available_tools_declaration(),
+        output=output,
+        **extra,
     )
 
     async def open_session() -> RealtimeConversationSession:  # noqa: C901
-        session: LMMSession = await session_scope.__aenter__()
+        session: ModelSession = await session_scope.__aenter__()
 
         async def handle_tool_request(
-            tool_request: LMMToolRequest,
+            tool_request: ModelToolRequest,
             /,
         ) -> None:
             try:
                 ctx.log_debug(
                     f"Handling tool ({tool_request.tool}) request ({tool_request.identifier}) ...",
                 )
-                response: LMMToolResponse = await toolbox.respond(tool_request)
+                response: ModelToolResponse = await toolbox.respond(tool_request)
                 if response.handling in ("completion", "extension"):
                     ctx.log_warning(
                         f"Tool handling `{response.handling}` is not supported in"
@@ -102,56 +94,47 @@ async def realtime_conversation_preparing(  # noqa: C901
                     f"...tool request ({tool_request.identifier}) handling completed!",
                 )
 
-        async def read() -> ConversationStreamElement:
+        async def read() -> ConversationOutputChunk | ConversationEvent:
             while True:
                 match await session.reading():
-                    case LMMStreamChunk() as chunk:
+                    case ModelOutputChunk() as chunk:
                         ctx.log_debug("...received response chunk...")
-                        return ConversationMessageChunk.model(
+                        return ConversationOutputChunk.of(
                             chunk.content,
-                            identifier=chunk.meta.identifier,
-                            message_identifier=chunk.meta.get_uuid(
-                                "message_identifier",
-                                default=uuid4(),
-                            ),
                             eod=chunk.eod,
                         )
 
-                    case LMMSessionEvent() as event:
+                    case ModelSessionEvent() as event:
                         ctx.log_debug(f"...received {event.category} event...")
                         return ConversationEvent.of(
                             event.category,
+                            content=event.content,
                             meta=event.meta,
                         )
 
-                    case LMMToolRequest() as tool_request:
+                    case ModelToolRequest() as tool_request:
                         ctx.log_debug(f"...received {tool_request.tool} request...")
                         ctx.spawn(
                             handle_tool_request,
                             tool_request,
                         )
-                        continue  # keep receiving elements
 
         async def write(
-            input: ConversationStreamElement,  # noqa: A002
+            input: ConversationInputChunk | ConversationEvent,  # noqa: A002
         ) -> None:
-            if isinstance(input, ConversationMessageChunk):
-                # pass chunk to LMM
+            if isinstance(input, ConversationEvent):
                 await session.writing(
-                    input=LMMStreamChunk.of(
-                        input.content,
-                        eod=input.eod,
-                        meta=input.meta,
+                    input=ModelSessionEvent.of(
+                        input.category,
+                        content=input.content,
                     )
                 )
 
             else:
-                assert isinstance(input, ConversationEvent)  # nosec: B101
-                # pass event to LMM
                 await session.writing(
-                    input=LMMSessionEvent.of(
-                        input.category,
-                        meta=input.meta,
+                    input=ModelInputChunk.of(
+                        input.content,
+                        eod=input.eod,
                     )
                 )
 
@@ -175,34 +158,3 @@ async def realtime_conversation_preparing(  # noqa: C901
         opening=open_session,
         closing=close_session,
     )
-
-
-def _to_lmm_recall(
-    recall: MemoryRecalling[Sequence[ConversationMessage]],
-    /,
-) -> MemoryRecalling[LMMContext]:
-    async def lmm_recall(
-        **extra: Any,
-    ) -> LMMContext:
-        return tuple(element.to_lmm_context() for element in await recall())
-
-    return lmm_recall
-
-
-def _to_lmm_remember(
-    remember: MemoryRemembering[ConversationMessage],
-    /,
-) -> MemoryRemembering[LMMContextElement]:
-    async def lmm_remember(
-        *items: LMMContextElement,
-        **extra: Any,
-    ) -> None:
-        await remember(
-            *(
-                ConversationMessage.from_lmm_context(element)
-                for element in items
-                if isinstance(element, LMMInput | LMMCompletion)
-            )
-        )
-
-    return lmm_remember
