@@ -1,17 +1,20 @@
+from draive.generation.audio import AudioGenerating, AudioGeneration
 from draive.generation.image import ImageGenerating, ImageGeneration
 from draive.generation.model import (
     ModelGenerating,
     ModelGeneration,
-    ModelGeneratorDecoder,
+    ModelGenerationDecoder,
 )
 from draive.generation.text import TextGenerating, TextGeneration
 
 __all__ = (
+    "AudioGenerating",
+    "AudioGeneration",
     "ImageGenerating",
     "ImageGeneration",
     "ModelGenerating",
     "ModelGeneration",
-    "ModelGeneratorDecoder",
+    "ModelGenerationDecoder",
     "TextGenerating",
     "TextGeneration",
 )
