@@ -1,5 +1,5 @@
 from draive.generation.image.state import ImageGeneration
-from draive.generation.image.typing import ImageGenerating
+from draive.generation.image.types import ImageGenerating
 
 __all__ = (
     "ImageGenerating",
