@@ -1,8 +1,8 @@
 from draive.generation.model.state import ModelGeneration
-from draive.generation.model.types import ModelGenerating, ModelGeneratorDecoder
+from draive.generation.model.types import ModelGenerating, ModelGenerationDecoder
 
 __all__ = (
     "ModelGenerating",
     "ModelGeneration",
-    "ModelGeneratorDecoder",
+    "ModelGenerationDecoder",
 )
