@@ -1,6 +1,6 @@
-from typing import Any, Self
+from typing import Any, Self, overload
 
-from haiway import State, ctx
+from haiway import State, statemethod
 
 from draive.evaluation import (
     EvaluatorResult,
@@ -41,25 +41,51 @@ class GuardrailsQualityVerification(State):
                 raise GuardrailsQualityException(
                     reason=result.evaluator,
                     content=content,
-                    meta=result.meta,
+                    meta={
+                        "performance": result.performance,
+                        "report": result.report(detailed=True),
+                    },
                 )
 
             else:
                 raise GuardrailsQualityException(
                     reason=result.scenario,
                     content=content,
-                    meta=result.meta,
+                    meta={
+                        "performance": result.performance,
+                        "report": result.report(detailed=True),
+                    },
                 )
 
         return cls(verifying=verifying)
 
+    @overload
     @classmethod
     async def verify(
         cls,
         content: Multimodal,
         /,
         **extra: Any,
+    ) -> None: ...
+
+    @overload
+    async def verify(
+        self,
+        content: Multimodal,
+        /,
+        **extra: Any,
+    ) -> None: ...
+
+    @statemethod
+    async def verify(
+        self,
+        content: Multimodal,
+        /,
+        **extra: Any,
     ) -> None:
-        await ctx.state(cls).verifying(MultimodalContent.of(content))
+        await self.verifying(
+            MultimodalContent.of(content),
+            **extra,
+        )
 
     verifying: GuardrailsQualityVerifying = _no_verification
