@@ -1,13 +1,13 @@
 from draive.helpers.instruction_preparation import (
     InstructionPreparationAmbiguity,
-    prepare_instruction,
+    prepare_instructions,
 )
-from draive.helpers.instruction_refinement import refine_instruction
+from draive.helpers.instruction_refinement import refine_instructions
 from draive.helpers.volatile_vector_index import VolatileVectorIndex
 
 __all__ = (
     "InstructionPreparationAmbiguity",
     "VolatileVectorIndex",
-    "prepare_instruction",
-    "refine_instruction",
+    "prepare_instructions",
+    "refine_instructions",
 )
