@@ -3,44 +3,46 @@ from collections.abc import Mapping, Sequence
 from typing import Any
 from uuid import UUID, uuid4
 
-from haiway import State, as_dict, ctx, execute_concurrently
+from haiway import Meta, State, as_dict, ctx, execute_concurrently
 
-from draive.evaluation import EvaluatorSuite, EvaluatorSuiteResult
-from draive.evaluation.suite import EvaluatorSuiteCase
-from draive.instructions import Instruction, Instructions
-from draive.multimodal import MultimodalContent, MultimodalTagElement
+from draive.evaluation import (
+    EvaluatorSuiteCase,
+    EvaluatorSuiteResult,
+    PreparedEvaluatorSuite,
+)
+from draive.models import Instructions, InstructionsRepository, ModelInstructions
+from draive.multimodal import MultimodalContent, MultimodalTag, TextContent
 from draive.parameters import DataModel
 from draive.stages import Stage, StageState, stage
 
-__all__ = ("refine_instruction",)
+__all__ = ("refine_instructions",)
 
 
-async def refine_instruction[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
-    instruction: Instruction,
+async def refine_instructions[Parameters: DataModel](
+    instructions: Instructions,
     /,
     *,
+    instructions_content: ModelInstructions | None = None,
     guidelines: str | None = None,
-    evaluation_suite: EvaluatorSuite[SuiteParameters, CaseParameters],
+    evaluator_suite: PreparedEvaluatorSuite[Parameters],
+    evaluator_cases: Sequence[EvaluatorSuiteCase[Parameters]],
     rounds_limit: int,
     sample_ratio: float = 0.1,
     candidates_limit: int = 3,
     performance_drop_threshold: float = 0.5,
     quality_threshold: float = 0.99,
     concurrent_nodes: int = 1,
-) -> Instruction:
+) -> ModelInstructions:
     """
-    Refine instruction using binary tree exploration with performance pruning.
+    Refine instructions using binary tree exploration with performance pruning.
 
     Each node generates exactly 2 complementary strategies.
     Branches are pruned if performance drops below threshold.
 
     Args:
-        instruction: The instruction to refine
+        instructions: The instructions to refine
         guidelines: Optional guidelines for refinement
-        evaluation_suite: Suite to evaluate instruction performance
+        evaluator_suite: Suite to evaluate instructions performance
         rounds_limit: Maximum depth of refinement tree
         sample_ratio: Fraction of passing cases to include in focused evaluation
         candidates_limit: Number of top candidates to fully evaluate
@@ -55,6 +57,7 @@ async def refine_instruction[
     assert 1 >= performance_drop_threshold > 0  # nosec: B101
     assert 1 >= quality_threshold >= 0  # nosec: B101
     assert concurrent_nodes > 0  # nosec: B101
+    assert len(evaluator_cases) > 0  # nosec: B101
 
     ctx.log_info(
         f"Starting tree-based refinement: "
@@ -65,68 +68,41 @@ async def refine_instruction[
 
     result: MultimodalContent = await Stage.sequence(
         _tree_initialization_stage(
-            instruction=instruction,
-            evaluation_suite=evaluation_suite,
+            instructions=instructions,
+            instructions_content=instructions_content,
+            evaluator_suite=evaluator_suite,
             sample_ratio=sample_ratio,
             performance_drop_threshold=performance_drop_threshold,
             guidelines=guidelines,
             rounds_limit=rounds_limit,
         ),
         _tree_exploration_stage(
-            evaluation_suite=evaluation_suite,
+            evaluator_suite=evaluator_suite,
+            evaluator_cases=evaluator_cases,
             rounds_limit=rounds_limit,
             quality_threshold=quality_threshold,
             concurrent_nodes=concurrent_nodes,
         ),
         _tree_finalization_stage(
-            evaluation_suite=evaluation_suite,
+            evaluator_suite=evaluator_suite,
             quality_threshold=quality_threshold,
             candidates_limit=candidates_limit,
         ),
     ).execute()
 
-    ctx.log_info("...instruction refinement finished!")
-    return instruction.updated(content=result.to_str())
+    ctx.log_info("...instructions refinement finished!")
+    return result.to_str()
 
 
-def _instructions(
-    *,
-    instruction: Instruction,
-) -> Instructions:
-    instructions: Instructions = ctx.state(Instructions)
-
-    async def instruction_fetch(
-        name: str,
-        /,
-        *,
-        arguments: Mapping[str, str | float | int] | None = None,
-        **extra: Any,
-    ) -> Instruction | None:
-        if name == instruction.name:
-            if arguments:
-                return instruction.updated(arguments={**instruction.arguments, **arguments})
-
-            else:
-                return instruction
-
-        else:  # preserve other prompts unchanged
-            return await instructions.fetch(
-                name,
-                arguments=arguments,
-                **extra,
-            )
-
-    return Instructions(fetching=instruction_fetch)
-
-
-class _RefinementTreeNode[SuiteParameters: DataModel, CaseParameters: DataModel](State):
+class _RefinementTreeNode(State):
     identifier: UUID
-    instruction: Instruction
+    instructions: Instructions
+    instructions_content: ModelInstructions
     strategy: str
     parent_id: UUID | None
     depth: int
-    focused_evaluation: EvaluatorSuiteResult[SuiteParameters, CaseParameters]
-    complete_evaluation: EvaluatorSuiteResult[SuiteParameters, CaseParameters] | None
+    focused_evaluation: EvaluatorSuiteResult
+    complete_evaluation: EvaluatorSuiteResult | None
     children: Sequence[UUID]
     pruned: bool
 
@@ -149,55 +125,81 @@ class _RefinementTreeNode[SuiteParameters: DataModel, CaseParameters: DataModel]
 
         return self.complete_evaluation.performance
 
+    @property
+    def patched_instructions_repository(self) -> InstructionsRepository:
+        repository: InstructionsRepository = ctx.state(InstructionsRepository)
 
-class _RefinementState[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](State):
+        async def instructions_loading(
+            name: str,
+            meta: Meta,
+            **extra: Any,
+        ) -> str | None:
+            # use updated instructions if able
+            if name == self.instructions.name:
+                return self.instructions_content
+
+            else:  # otherwise preserve unchanged instructions
+                return await repository.loading(
+                    name=name,
+                    meta=meta,
+                    **extra,
+                )
+
+        return InstructionsRepository(
+            listing=repository.listing,  # keep current listing
+            loading=instructions_loading,  # replace loading
+            # do not allow modifications - use default noop implementation
+        )
+
+
+class _RefinementState(State):
     root: _RefinementTreeNode
-    nodes: Mapping[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]]
+    nodes: Mapping[UUID, _RefinementTreeNode]
     sample_ratio: float
     performance_drop_threshold: float
     guidelines: str | None = None
     rounds_remaining: int
 
-    def leafs(self) -> Sequence[_RefinementTreeNode[SuiteParameters, CaseParameters]]:
+    def leaves(self) -> Sequence[_RefinementTreeNode]:
         return [node for node in self.nodes.values() if node.is_leaf and not node.pruned]
 
 
-def _select_focused_cases[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+def _select_focused_cases[Parameters: DataModel](
     *,
-    evaluation_result: EvaluatorSuiteResult[SuiteParameters, CaseParameters],
-    evaluation_cases: Sequence[EvaluatorSuiteCase[CaseParameters]],
+    evaluation_result: EvaluatorSuiteResult,
+    evaluator_cases: Sequence[EvaluatorSuiteCase[Parameters]],
     sample_ratio: float,
-) -> Sequence[EvaluatorSuiteCase[CaseParameters]]:
+) -> Sequence[str]:
     # Get all failing cases
-    failing_cases: Sequence[EvaluatorSuiteCase[CaseParameters]] = [
-        case_result.case for case_result in evaluation_result.cases if not case_result.passed
+    failing_cases: Sequence[str] = [
+        case_result.case_identifier
+        for case_result in evaluation_result.results
+        if not case_result.passed
     ]
 
-    # Get passing cases and sample
-    passing_cases: Sequence[EvaluatorSuiteCase[CaseParameters]] = [
-        case_result.case for case_result in evaluation_result.cases if case_result.passed
+    # Get passing cases
+    passing_cases: Sequence[str] = [
+        case_result.case_identifier
+        for case_result in evaluation_result.results
+        if case_result.passed
     ]
 
     # Get other, previously excluded cases
-    additional_cases: Sequence[EvaluatorSuiteCase[CaseParameters]] = [
-        case for case in evaluation_cases if case not in evaluation_result.cases
+    additional_cases: Sequence[str] = [
+        case.identifier
+        for case in evaluator_cases
+        if case.identifier not in failing_cases and case.identifier not in passing_cases
     ]
 
-    # Intelligent sampling: sample some passing cases
-    sampling_cases_pool: Sequence[EvaluatorSuiteCase[CaseParameters]] = (
-        passing_cases + additional_cases
-    )
+    sampling_cases_pool: Sequence[str] = passing_cases + additional_cases
     sample_size: int = (
         max(1, int(len(sampling_cases_pool) * sample_ratio)) if sampling_cases_pool else 0
     )
-    sampling_cases: Sequence[EvaluatorSuiteCase[CaseParameters]] = (
-        random.sample(sampling_cases_pool, min(len(sampling_cases_pool), sample_size))  # nosec: B311
+    sampling_cases: Sequence[str] = (
+        random.sample(  # nosec: B311
+            sampling_cases_pool,
+            min(len(sampling_cases_pool), sample_size),
+        )
         if sample_size > 0
         else []
     )
@@ -212,13 +214,11 @@ def _select_focused_cases[
     return failing_cases + sampling_cases
 
 
-def _tree_initialization_stage[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+def _tree_initialization_stage[Parameters: DataModel](
     *,
-    instruction: Instruction,
-    evaluation_suite: EvaluatorSuite[SuiteParameters, CaseParameters],
+    instructions: Instructions,
+    instructions_content: ModelInstructions | None,
+    evaluator_suite: PreparedEvaluatorSuite[Parameters],
     sample_ratio: float,
     performance_drop_threshold: float,
     guidelines: str | None,
@@ -229,18 +229,24 @@ def _tree_initialization_stage[
         *,
         state: StageState,
     ) -> StageState:
-        ctx.log_info("...evaluating initial instruction with full suite...")
-        # Make initial evaluation
-        evaluation: EvaluatorSuiteResult[SuiteParameters, CaseParameters]
-        with ctx.updated(_instructions(instruction=instruction)):
-            evaluation = await evaluation_suite()
+        ctx.log_info("...evaluating initial instructions with full suite...")
+        # Make initial evaluation using current instructions
+        evaluation: EvaluatorSuiteResult = await evaluator_suite()
 
         ctx.log_info(f"...initial score: {evaluation.performance:.2f}...")
 
+        content: ModelInstructions
+        if instructions_content is None:
+            content = await InstructionsRepository.load(instructions)
+
+        else:
+            content = instructions_content
+
         # Create root node
         root_node = _RefinementTreeNode(
             identifier=uuid4(),
-            instruction=instruction,
+            instructions=instructions,
+            instructions_content=content,
             strategy="initial",
             parent_id=None,
             depth=0,
@@ -253,7 +259,7 @@ def _tree_initialization_stage[
 
         # Setup initial state
         return state.updated(
-            _RefinementState[SuiteParameters, CaseParameters](
+            _RefinementState(
                 root=root_node,
                 nodes={root_node.identifier: root_node},
                 sample_ratio=sample_ratio,
@@ -261,19 +267,17 @@ def _tree_initialization_stage[
                 guidelines=guidelines,
                 rounds_remaining=rounds_limit,
             ),
-            # set the result to be initial instruction
-            result=MultimodalContent.of(instruction.content),
+            # set the result to be initial instruction content
+            result=MultimodalContent.of(content),
         )
 
     return tree_initialization
 
 
-def _tree_exploration_stage[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+def _tree_exploration_stage[Parameters: DataModel](
     *,
-    evaluation_suite: EvaluatorSuite[SuiteParameters, CaseParameters],
+    evaluator_suite: PreparedEvaluatorSuite[Parameters],
+    evaluator_cases: Sequence[EvaluatorSuiteCase[Parameters]],
     rounds_limit: int,
     quality_threshold: float,
     concurrent_nodes: int,
@@ -287,33 +291,26 @@ def _tree_exploration_stage[
             _RefinementState,
             required=True,
         )
-        evaluation_cases: Sequence[
-            EvaluatorSuiteCase[CaseParameters]
-        ] = await evaluation_suite.cases()
 
         async def explore(
-            node: _RefinementTreeNode[SuiteParameters, CaseParameters],
-        ) -> Mapping[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]]:
+            node: _RefinementTreeNode,
+        ) -> Mapping[UUID, _RefinementTreeNode]:
             return await _explore_node(
                 node=node,
-                evaluation_suite=evaluation_suite,
-                evaluation_cases=evaluation_cases,
+                evaluator_suite=evaluator_suite,
+                evaluator_cases=evaluator_cases,
                 sample_ratio=refinement_state.sample_ratio,
                 performance_drop_threshold=refinement_state.performance_drop_threshold,
                 guidelines=refinement_state.guidelines,
             )
 
-        node_updates: Sequence[
-            Mapping[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]]
-        ] = await execute_concurrently(
+        node_updates: Sequence[Mapping[UUID, _RefinementTreeNode]] = await execute_concurrently(
             explore,
-            refinement_state.leafs(),
+            refinement_state.leaves(),
             concurrent_tasks=concurrent_nodes,
         )
 
-        updated_nodes: dict[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]] = as_dict(
-            refinement_state.nodes
-        )
+        updated_nodes: dict[UUID, _RefinementTreeNode] = as_dict(refinement_state.nodes)
         for nodes in node_updates:
             updated_nodes = {
                 **updated_nodes,
@@ -364,17 +361,14 @@ def _tree_exploration_stage[
     )
 
 
-async def _explore_node[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
-    node: _RefinementTreeNode[SuiteParameters, CaseParameters],
-    evaluation_suite: EvaluatorSuite[SuiteParameters, CaseParameters],
-    evaluation_cases: Sequence[EvaluatorSuiteCase[CaseParameters]],
+async def _explore_node[Parameters: DataModel](
+    node: _RefinementTreeNode,
+    evaluator_suite: PreparedEvaluatorSuite[Parameters],
+    evaluator_cases: Sequence[EvaluatorSuiteCase[Parameters]],
     sample_ratio: float,
     performance_drop_threshold: float,
     guidelines: str | None,
-) -> Mapping[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]]:
+) -> Mapping[UUID, _RefinementTreeNode]:
     assert not node.pruned  # nosec: B101 # skip pruned branches
     ctx.log_info(
         f"Exploring node {node.identifier} at"
@@ -382,27 +376,45 @@ async def _explore_node[
     )
 
     # Generate exactly 2 complementary strategies
-    strategies: Sequence[tuple[str, Instruction]] = await _generate_refined_instructions(
-        instruction=node.instruction,
+    strategies: Sequence[tuple[str, ModelInstructions]] = await _generate_refined_instructions(
+        current_content=node.instructions_content,
         evaluation_result=node.focused_evaluation,
         parent_strategy=node.strategy,
         guidelines=guidelines,
     )
 
     # Create and evaluate child nodes
-    focused_suite_cases: Sequence[EvaluatorSuiteCase[CaseParameters]] = _select_focused_cases(
+    focused_suite_cases: Sequence[str] = _select_focused_cases(
         evaluation_result=node.focused_evaluation,
-        evaluation_cases=evaluation_cases,
+        evaluator_cases=evaluator_cases,
         sample_ratio=sample_ratio,
     )
 
-    children: dict[UUID, _RefinementTreeNode[SuiteParameters, CaseParameters]] = {}
-    for strategy_name, refined_instruction in strategies:
+    children: dict[UUID, _RefinementTreeNode] = {}
+    for strategy_name, refined_instructions in strategies:
+        # Create child node
+        child_node = _RefinementTreeNode(
+            identifier=uuid4(),
+            instructions=node.instructions,
+            instructions_content=refined_instructions,
+            strategy=strategy_name,
+            parent_id=node.identifier,
+            depth=node.depth + 1,
+            # initialize with placeholder
+            focused_evaluation=EvaluatorSuiteResult(
+                suite="placeholder",
+                results=(),
+            ),
+            complete_evaluation=None,
+            children=(),
+            pruned=False,
+        )
+
         # Evaluate with focused suite
         ctx.log_info(f"Evaluating strategy '{strategy_name}'...")
-        focused_evaluation: EvaluatorSuiteResult[SuiteParameters, CaseParameters]
-        with ctx.updated(_instructions(instruction=refined_instruction)):
-            focused_evaluation = await evaluation_suite(focused_suite_cases)
+        focused_evaluation: EvaluatorSuiteResult
+        with ctx.updated(child_node.patched_instructions_repository):
+            focused_evaluation = await evaluator_suite(focused_suite_cases)
 
         # Check for performance drop
         performance_ratio = (
@@ -412,19 +424,11 @@ async def _explore_node[
         )
         pruned: bool = performance_ratio < performance_drop_threshold
 
-        # Create child node
-        child_node = _RefinementTreeNode(
-            identifier=uuid4(),
-            instruction=refined_instruction,
-            strategy=strategy_name,
-            parent_id=node.identifier,
-            depth=node.depth + 1,
+        # update node with evaluation data
+        children[child_node.identifier] = child_node.updated(
             focused_evaluation=focused_evaluation,
-            complete_evaluation=None,
-            children=(),
             pruned=pruned,
         )
-        children[child_node.identifier] = child_node
 
         if pruned:
             ctx.log_warning(
@@ -435,24 +439,21 @@ async def _explore_node[
         else:
             ctx.log_info(
                 f"Created child node {child_node.identifier}: strategy={strategy_name}, "
-                f"score={child_node.focused_evaluation_performance:.2f}"
+                f"score={focused_evaluation.performance:.2f}"
                 f" ({performance_ratio:.1%} of parent)"
             )
 
     return children
 
 
-async def _generate_strategy_metadata[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+async def _generate_strategy_metadata(
     *,
-    evaluation_result: EvaluatorSuiteResult[SuiteParameters, CaseParameters],
+    evaluation_result: EvaluatorSuiteResult,
     parent_strategy: str | None,
     guidelines: str | None,
 ) -> Sequence[tuple[str, str]]:
     """
-    Generate strategy metadata (name and approach) without instruction content.
+    Generate strategy metadata (name and approach) without instructions content.
 
     Returns:
         Sequence of tuples containing (name, approach) for each strategy.
@@ -464,12 +465,12 @@ async def _generate_strategy_metadata[
     )
 
     strategy_prompt: str = f"""
-You are an expert prompt engineer providing feedback and recemmondations for instruction improvement.
+You are an expert prompt engineer providing feedback and recommendations for instructions improvement.
 Based on the evaluation failures analysis, prepare EXACTLY 2 DIFFERENT refinement strategies.
 
 <previous_strategy>{parent_strategy or "Initial"}</previous_strategy>
 <task>
-Generate 2 instruction refinement strategies and recommendations that:
+Generate 2 instructions refinement strategies and recommendations that:
 1. Take CONTRASTING approaches to fixing the issues
 2. Address different aspects of the failures
 3. Are mutually exclusive in their approach
@@ -477,11 +478,11 @@ Generate 2 instruction refinement strategies and recommendations that:
 {f"\n<guidelines>\n{guidelines}</guidelines>\n" if guidelines else ""}
 <examples>
 <strategy>
-<name>Examplification</name>
+<name>Exemplification</name>
 <approach>Add more specific details and examples</approach>
 </strategy>
 <strategy>
-<name>Constraning</name>
+<name>Constraining</name>
 <approach>Add explicit constraints and rules</approach>
 </strategy>
 </examples>
@@ -496,23 +497,14 @@ For each strategy, provide the name and approach in the following tags format:
 
     response: MultimodalContent = await Stage.completion(
         f"<failure_analysis>\n{failure_report}\n</failure_analysis>",
-        instruction=strategy_prompt,
+        instructions=strategy_prompt,
     ).execute()
 
     # Parse strategies
     strategies: list[tuple[str, str]] = []
-    for strategy_element in MultimodalTagElement.parse(
-        "strategy",
-        content=response,
-    ):
-        name_element: MultimodalTagElement | None = MultimodalTagElement.parse_first(
-            "name",
-            content=strategy_element.content,
-        )
-        approach_element: MultimodalTagElement | None = MultimodalTagElement.parse_first(
-            "approach",
-            content=strategy_element.content,
-        )
+    for strategy_element in response.tags("strategy"):
+        name_element: MultimodalTag | None = strategy_element.content.tag("name")
+        approach_element: MultimodalTag | None = strategy_element.content.tag("approach")
 
         if name_element and approach_element:
             strategy_name: str = name_element.content.to_str().strip()
@@ -522,17 +514,14 @@ For each strategy, provide the name and approach in the following tags format:
     return strategies
 
 
-async def _generate_instruction_content[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+async def _generate_instructions(
     *,
-    current_instruction_content: str,
+    current_content: ModelInstructions,
     strategy_name: str,
     strategy_approach: str,
-    evaluation_result: EvaluatorSuiteResult[SuiteParameters, CaseParameters],
+    evaluation_result: EvaluatorSuiteResult,
     guidelines: str | None,
-) -> str:
+) -> ModelInstructions:
     # Analyze failures
     failure_report: str = evaluation_result.report(
         detailed=True,
@@ -542,8 +531,8 @@ async def _generate_instruction_content[
     refinement_prompt: str = f"""\
 You are an expert prompt engineer refining provided instructions referred as a subject.
 Provide an updated version of the subject that implements the described strategy to address the evaluation failures.
-Strictly focus on delivering the best possible instruction content with excellent clarity and performance.
-Ensure to apply modifications by following requested strategy while keeping original instruction intent, goal and formatting style.
+Strictly focus on delivering the best possible instructions content with excellent clarity and performance.
+Ensure to apply modifications by following requested strategy while keeping original instructions intent, goal and formatting style.
 
 <strategy>
 <name>
@@ -555,36 +544,35 @@ Ensure to apply modifications by following requested strategy while keeping orig
 </strategy>
 
 <subject>
-{current_instruction_content}
+{current_content}
 </subject>
 {f"\n<guidelines>\n{guidelines}</guidelines>\n" if guidelines else ""}
 <format>
-Provide ONLY the refined instruction content, without any explanation or metadata.
+Provide ONLY the refined instructions content, without any explanation or metadata.
 </format>
 """  # noqa: E501
 
-    ctx.log_info(f"Generating updated instruction using {strategy_name}:\n\n{strategy_approach}")
+    ctx.log_info(f"Generating updated instructions using {strategy_name}:\n\n{strategy_approach}")
     response: MultimodalContent = await Stage.completion(
         f"<failure_analysis>\n{failure_report}\n</failure_analysis>",
-        instruction=refinement_prompt,
+        instructions=refinement_prompt,
     ).execute()
 
     updated_instruction: str = response.to_str().strip()
 
-    ctx.log_info(f"Prepared updated instruction using {strategy_name}:\n\n{updated_instruction}")
+    ctx.log_info(f"Prepared updated instructions using {strategy_name}")
+    ctx.log_debug(f"Updated instruction for {strategy_name}: {updated_instruction}")
 
     return updated_instruction
 
 
-async def _generate_refined_instructions[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
-    instruction: Instruction,
-    evaluation_result: EvaluatorSuiteResult[SuiteParameters, CaseParameters],
+async def _generate_refined_instructions(
+    *,
+    current_content: ModelInstructions,
+    evaluation_result: EvaluatorSuiteResult,
     parent_strategy: str | None,
     guidelines: str | None,
-) -> Sequence[tuple[str, Instruction]]:
+) -> Sequence[tuple[str, ModelInstructions]]:
     # Step 1: Generate strategy metadata
     strategy_metadata: Sequence[tuple[str, str]] = await _generate_strategy_metadata(
         evaluation_result=evaluation_result,
@@ -592,18 +580,17 @@ async def _generate_refined_instructions[
         guidelines=guidelines,
     )
 
-    # Step 2: Generate instruction content for each strategy
-    refined_strategies: list[tuple[str, Instruction]] = []
+    # Step 2: Generate instructions content for each strategy
+    refined_strategies: list[tuple[str, ModelInstructions]] = []
     for strategy_name, strategy_approach in strategy_metadata:
-        refined_content = await _generate_instruction_content(
-            current_instruction_content=instruction.content,
+        refined_instructions = await _generate_instructions(
+            current_content=current_content,
             strategy_name=strategy_name,
             strategy_approach=strategy_approach,
             evaluation_result=evaluation_result,
             guidelines=guidelines,
         )
-        refined_instruction: Instruction = instruction.updated(content=refined_content)
-        refined_strategies.append((strategy_name, refined_instruction))
+        refined_strategies.append((strategy_name, refined_instructions))
 
     # Validate and return results
     match len(refined_strategies):
@@ -623,12 +610,9 @@ async def _generate_refined_instructions[
             return refined_strategies[:2]
 
 
-def _tree_finalization_stage[
-    SuiteParameters: DataModel,
-    CaseParameters: DataModel,
-](
+def _tree_finalization_stage[Parameters: DataModel](
     *,
-    evaluation_suite: EvaluatorSuite[SuiteParameters, CaseParameters],
+    evaluator_suite: PreparedEvaluatorSuite[Parameters],
     quality_threshold: float,
     candidates_limit: int,
 ) -> Stage:
@@ -655,7 +639,7 @@ def _tree_finalization_stage[
         ctx.log_info(f"Running full evaluation on top {len(candidates)} candidates")
 
         # Full evaluation of top candidates
-        best_instruction: Instruction = refinement_state.root.instruction
+        best_instructions: ModelInstructions = refinement_state.root.instructions_content
         best_score: float = refinement_state.root.complete_evaluation_performance or 0
         best_node: _RefinementTreeNode = refinement_state.root
 
@@ -665,9 +649,9 @@ def _tree_finalization_stage[
                 f"(strategy: {candidate_node.strategy}, depth: {candidate_node.depth})"
             )
 
-            complete_evaluation: EvaluatorSuiteResult[SuiteParameters, CaseParameters]
-            with ctx.updated(_instructions(instruction=candidate_node.instruction)):
-                complete_evaluation = await evaluation_suite()
+            complete_evaluation: EvaluatorSuiteResult
+            with ctx.updated(candidate_node.patched_instructions_repository):
+                complete_evaluation = await evaluator_suite()
 
             # Update node with full eval score
             updated_node: _RefinementTreeNode = candidate_node.updated(
@@ -687,7 +671,7 @@ def _tree_finalization_stage[
             )
 
             if complete_evaluation.performance > best_score:
-                best_instruction = updated_node.instruction
+                best_instructions = updated_node.instructions_content
                 best_score = complete_evaluation.performance
                 best_node = updated_node
 
@@ -697,12 +681,14 @@ def _tree_finalization_stage[
             best_node=best_node,
         )
 
-        # Update result with best instruction and updated state
+        # Update result with best instructions and updated state
         return state.updated(
             refinement_state,
             result=MultimodalContent.of(
-                best_instruction.content,
-                meta={"score": best_score},
+                TextContent.of(
+                    best_instructions,
+                    meta={"score": best_score},
+                ),
             ),
         )
 
