@@ -4,12 +4,12 @@ from collections.abc import AsyncGenerator, Callable, Collection, Coroutine, Map
 from contextlib import AbstractAsyncContextManager, asynccontextmanager
 from itertools import chain
 from types import TracebackType
-from typing import Any, Literal, Self, cast, final
+from typing import Any, Self, cast, final
 from urllib.parse import ParseResult, urlparse, urlunparse
 from uuid import uuid4
 
-from haiway import Meta, MetaTags, as_dict, as_list, ctx
-from mcp import ClientSession, GetPromptResult, ListToolsResult, StdioServerParameters, stdio_client
+from haiway import BasicValue, Meta, MetaTags, as_dict, as_list, ctx
+from mcp import ClientSession, ListToolsResult, StdioServerParameters, stdio_client
 from mcp import Tool as MCPTool
 from mcp.client.sse import sse_client
 from mcp.types import AudioContent as MCPAudioContent
@@ -26,12 +26,17 @@ from mcp.types import ResourceLink as MCPResourceLink
 from mcp.types import TextContent as MCPTextContent
 from pydantic import AnyUrl
 
-from draive.lmm import LMMCompletion, LMMContextElement, LMMInput
-from draive.multimodal import MediaData, MultimodalContent, TextContent
-from draive.parameters import BasicValue, DataModel, validated_specification
-from draive.prompts import Prompt, PromptDeclaration, PromptDeclarationArgument, Prompts
-from draive.resources import Resource, ResourceContent, ResourceDeclaration, Resources
-from draive.tools import FunctionTool, Tool, ToolError, Tools
+from draive.models import (
+    FunctionTool,
+    ModelToolParametersSpecification,
+    ModelToolSpecification,
+    Tool,
+    ToolError,
+    ToolsProvider,
+)
+from draive.multimodal import ArtifactContent, MultimodalContent, TextContent
+from draive.parameters import DataModel
+from draive.resources import ResourceContent, ResourceReference, ResourcesRepository
 
 __all__ = (
     "MCPClient",
@@ -49,7 +54,7 @@ class MCPClient:
         command: str,
         args: Sequence[str] | None = None,
         env: Mapping[str, str] | None = None,
-        features: Collection[Literal["resources", "prompts", "tools"]] | None = None,
+        features: Collection[type[ResourcesRepository] | type[ToolsProvider]] | None = None,
         tags: MetaTags | None = None,
     ) -> Self:
         @asynccontextmanager
@@ -67,7 +72,7 @@ class MCPClient:
         return cls(
             identifier or uuid4().hex,
             session_manager=mcp_stdio_session(),
-            features=features if features is not None else {"resources", "prompts", "tools"},
+            features=features if features is not None else (ResourcesRepository, ToolsProvider),
             tags=tags if tags is not None else (),
         )
 
@@ -80,7 +85,7 @@ class MCPClient:
         headers: Mapping[str, Any] | None = None,
         timeout: float = 5,
         sse_read_timeout: float = 60 * 5,
-        features: Collection[Literal["resources", "prompts", "tools"]] | None = None,
+        features: Collection[type[ResourcesRepository] | type[ToolsProvider]] | None = None,
         tags: MetaTags | None = None,
     ) -> Self:
         @asynccontextmanager
@@ -97,7 +102,7 @@ class MCPClient:
         return cls(
             identifier or uuid4().hex,
             session_manager=mcp_sse_session(),
-            features=features if features is not None else {"resources", "prompts", "tools"},
+            features=features if features is not None else (ResourcesRepository, ToolsProvider),
             tags=tags if tags is not None else (),
         )
 
@@ -114,19 +119,19 @@ class MCPClient:
         identifier: str,
         *,
         session_manager: AbstractAsyncContextManager[ClientSession],
-        features: Collection[Literal["resources", "prompts", "tools"]],
+        features: Collection[type[ResourcesRepository] | type[ToolsProvider]],
         tags: MetaTags,
     ) -> None:
         self.identifier: str = identifier
         self._session_manager: AbstractAsyncContextManager[ClientSession] = session_manager
         self._session: ClientSession
-        self._features: Collection[Literal["resources", "prompts", "tools"]] = features
+        self._features: Collection[type[ResourcesRepository] | type[ToolsProvider]] = features
         self.tags: MetaTags = tags
 
     async def resources_list(
         self,
         **extra: Any,
-    ) -> Sequence[ResourceDeclaration]:
+    ) -> Sequence[ResourceReference]:
         assert hasattr(  # nosec: B101
             self,
             "_session",
@@ -137,15 +142,15 @@ class MCPClient:
         result: ListResourcesResult = await self._session.list_resources()
 
         return [
-            ResourceDeclaration(
+            ResourceReference(
                 uri=self._with_uri_identifier(resource.uri.unicode_string()),
-                name=resource.name,
-                description=resource.description,
                 mime_type=resource.mimeType,
                 meta=Meta(
                     {
                         "mcp_server": self.identifier,
                         "tags": self.tags,
+                        "name": resource.name,
+                        "description": resource.description,
                     }
                 ),
             )
@@ -156,122 +161,51 @@ class MCPClient:
         self,
         uri: str,
         **extra: Any,
-    ) -> Resource | None:
+    ) -> Collection[ResourceReference] | ResourceContent | None:
         assert hasattr(  # nosec: B101
             self,
             "_session",
         ), "MCPClient has to be initialized through async context entering"
+
         result: ReadResourceResult = await self._session.read_resource(
             uri=AnyUrl(self._without_uri_identifier(uri))
         )
 
-        match [self._convert_resource_content(element) for element in result.contents]:
+        match result.contents:
             case [resource]:
                 # if there is only a single element return it directly
-                return resource
+                return self._convert_resource_content(resource)
 
             case [*resources]:
-                return Resource(
-                    uri=uri,
-                    name="resources",
-                    description=None,
-                    content=resources,
-                    meta=Meta(
-                        {
-                            "mcp_server": self.identifier,
-                            "tags": self.tags,
-                        }
-                    ),
-                )
+                # otherwise convert to references ignoring content
+                return [
+                    ResourceReference(
+                        uri=resource.uri.unicode_string(),
+                        mime_type=resource.mimeType,
+                        meta=Meta(
+                            {
+                                "mcp_server": self.identifier,
+                                "tags": self.tags,
+                            }
+                        ),
+                    )
+                    for resource in resources
+                ]
 
     async def resource_upload(
         self,
-        resource: Resource,
+        uri: str,
+        content: ResourceContent,
         **extra: Any,
-    ) -> None:
+    ) -> Meta:
         raise NotImplementedError("Resource uploading is not supported by MCP servers")
 
-    async def prompts_list(
+    async def resource_delete(
         self,
+        uri: str,
         **extra: Any,
-    ) -> Sequence[PromptDeclaration]:
-        assert hasattr(  # nosec: B101
-            self,
-            "_session",
-        ), "MCPClient has to be initialized through async context entering"
-
-        # TODO: in theory there are some extra elements like pagination
-        # we are not supporting it as for now (how to pass cursor anyways?)
-        return tuple(
-            PromptDeclaration(
-                name=self._with_prompt_name_identifier(prompt.name),
-                description=prompt.description,
-                arguments=tuple(
-                    PromptDeclarationArgument(
-                        name=argument.name,
-                        specification={
-                            "type": "string",
-                        }
-                        if argument.description is None
-                        else {
-                            "type": "string",
-                            "description": argument.description,
-                        },
-                        required=argument.required if argument.required is not None else True,
-                    )
-                    for argument in prompt.arguments
-                )
-                if prompt.arguments
-                else (),
-                meta=Meta(
-                    {
-                        "mcp_server": self.identifier,
-                        "tags": self.tags,
-                    }
-                ),
-            )
-            for prompt in (await self._session.list_prompts()).prompts
-        )
-
-    async def prompt_fetch(
-        self,
-        name: str,
-        *,
-        arguments: Mapping[str, str] | None,
-        **extra: Any,
-    ) -> Prompt:
-        assert hasattr(  # nosec: B101
-            self, "_session"
-        ), "MCPClient has to be initialized through async context entering"
-
-        fetched_prompt: GetPromptResult = await self._session.get_prompt(
-            name=self._without_prompt_name_identifier(name),
-            arguments=as_dict(arguments) if arguments is not None else None,
-        )
-
-        prompt_context: list[LMMContextElement] = []
-        for element in fetched_prompt.messages:
-            match element.role:
-                case "user":
-                    prompt_context.append(LMMInput.of(await _convert_content(element.content)))
-
-                case "assistant":
-                    prompt_context.append(LMMCompletion.of(await _convert_content(element.content)))
-
-                case _:
-                    raise ValueError(f"Unsupported prompt element role: {element.role}")
-
-        return Prompt(
-            name=name,
-            description=fetched_prompt.description,
-            content=prompt_context,
-            meta=Meta(
-                {
-                    "mcp_server": self.identifier,
-                    "tags": self.tags,
-                }
-            ),
-        )
+    ) -> None:
+        raise NotImplementedError("Resource deleting is not supported by MCP servers")
 
     async def tools_fetch(
         self,
@@ -310,35 +244,11 @@ class MCPClient:
 
         return content
 
-    def _with_prompt_name_identifier(
-        self,
-        name: str,
-        /,
-    ) -> str:
-        """Add server identifier to prompt name."""
-
-        return f"{self.identifier}|{name}"
-
-    def _without_prompt_name_identifier(
-        self,
-        name: str,
-        /,
-    ) -> str:
-        """Remove server identifier from prompt name."""
-
-        match name.split("|", 1):
-            case [identifier, reminder] if identifier == self.identifier:
-                return reminder
-
-            case _:
-                return name
-
     def _with_uri_identifier(
         self,
         uri: str,
         /,
     ) -> str:
-        """Add server identifier to URI."""
         if not uri:
             return uri
 
@@ -363,7 +273,7 @@ class MCPClient:
             return urlunparse(
                 (
                     # default to mcp scheme if empty
-                    parsed.scheme or "mcpclient",
+                    parsed.scheme or "mcp",
                     # use identifier as domain/netloc
                     self.identifier,
                     path,
@@ -378,7 +288,6 @@ class MCPClient:
         uri: str,
         /,
     ) -> str:
-        """Remove server identifier from URI."""
         if not uri:
             return uri
 
@@ -389,7 +298,7 @@ class MCPClient:
         if parsed.netloc == self.identifier:
             return urlunparse(
                 (
-                    parsed.scheme if parsed.scheme != "mcpclient" else "",
+                    parsed.scheme if parsed.scheme != "mcp" else "",
                     "",
                     parsed.path,
                     parsed.params,
@@ -418,17 +327,12 @@ class MCPClient:
         self,
         resource: TextResourceContents | BlobResourceContents,
         /,
-    ) -> Resource:
+    ) -> ResourceContent:
         match resource:
             case TextResourceContents() as text_resource:
-                return Resource(
-                    uri=self._with_uri_identifier(resource.uri.unicode_string()),
-                    name="resource",
-                    description=None,
-                    content=ResourceContent(
-                        blob=text_resource.text.encode(),
-                        mime_type=text_resource.mimeType or "text/plain",
-                    ),
+                return ResourceContent(
+                    data=urlsafe_b64decode(text_resource.text.encode()).decode(),
+                    mime_type=text_resource.mimeType or "text/plain",
                     meta=Meta(
                         {
                             "mcp_server": self.identifier,
@@ -438,14 +342,9 @@ class MCPClient:
                 )
 
             case BlobResourceContents() as blob_resource:
-                return Resource(
-                    uri=self._with_uri_identifier(resource.uri.unicode_string()),
-                    name="resource",
-                    description=None,
-                    content=ResourceContent(
-                        blob=urlsafe_b64decode(blob_resource.blob),
-                        mime_type=blob_resource.mimeType or "application/octet-stream",
-                    ),
+                return ResourceContent(
+                    data=blob_resource.blob,
+                    mime_type=blob_resource.mimeType or "application/octet-stream",
                     meta=Meta(
                         {
                             "mcp_server": self.identifier,
@@ -454,17 +353,18 @@ class MCPClient:
                     ),
                 )
 
-    async def __aenter__(self) -> Sequence[Resources | Prompts | Tools]:
+    async def __aenter__(self) -> Sequence[ResourcesRepository | ToolsProvider]:
         self._session = await self._session_manager.__aenter__()
         await self._session.initialize()
 
-        features: list[Resources | Prompts | Tools] = []
-        if "resources" in self._features:
+        features: list[ResourcesRepository | ToolsProvider] = []
+        if ResourcesRepository in self._features:
             features.append(
-                Resources(
+                ResourcesRepository(
                     list_fetching=self.resources_list,
                     fetching=self.resource_fetch,
                     uploading=self.resource_upload,
+                    deleting=self.resource_delete,
                     meta=Meta(
                         {
                             "mcp_server": self.identifier,
@@ -474,23 +374,10 @@ class MCPClient:
                 )
             )
 
-        if "prompts" in self._features:
+        if ToolsProvider in self._features:
             features.append(
-                Prompts(
-                    list_fetching=self.prompts_list,
-                    fetching=self.prompt_fetch,
-                    meta=Meta(
-                        {
-                            "mcp_server": self.identifier,
-                            "tags": self.tags,
-                        }
-                    ),
-                )
-            )
-        if "tools" in self._features:
-            features.append(
-                Tools(
-                    fetching=self.tools_fetch,
+                ToolsProvider(
+                    loading=self.tools_fetch,
                     meta=Meta(
                         {
                             "mcp_server": self.identifier,
@@ -530,17 +417,17 @@ async def _convert_content(  # noqa: C901, PLR0911
 
         case MCPImageContent() as image:
             return MultimodalContent.of(
-                MediaData.of(
+                ResourceContent.of(
                     urlsafe_b64decode(image.data),
-                    media=image.mimeType,
+                    mime_type=image.mimeType,
                 )
             )
 
         case MCPAudioContent() as audio:
             return MultimodalContent.of(
-                MediaData.of(
+                ResourceContent.of(
                     urlsafe_b64decode(audio.data),
-                    media=audio.mimeType,
+                    mime_type=audio.mimeType,
                 )
             )
 
@@ -553,7 +440,7 @@ async def _convert_content(  # noqa: C901, PLR0911
                     match blob.mimeType:
                         case None:
                             raise NotImplementedError(
-                                "Unsuppoprted MCP prompt message element - missing mime!"
+                                "Unsupported embedded resource - missing mime!"
                             )
 
                         case "text/plain":
@@ -563,15 +450,17 @@ async def _convert_content(  # noqa: C901, PLR0911
 
                         case "application/json":
                             return MultimodalContent.of(
-                                DataModel.from_json(urlsafe_b64decode(blob.blob).decode())
+                                ArtifactContent.of(
+                                    DataModel.from_json(urlsafe_b64decode(blob.blob).decode())
+                                )
                             )
 
                         case other:
                             # try to match supported media or raise an exception
                             return MultimodalContent.of(
-                                MediaData.of(
+                                ResourceContent.of(
                                     urlsafe_b64decode(blob.blob),
-                                    media=other,
+                                    mime_type=other,
                                 )
                             )
 
@@ -598,12 +487,18 @@ def _convert_tool(
     return FunctionTool(
         name=name,
         description=mcp_tool.description,
-        parameters=validated_specification(mcp_tool.inputSchema),
+        parameters=cast(
+            ModelToolParametersSpecification,
+            {
+                **mcp_tool.inputSchema,
+                "additionalProperties": False,
+            },
+        ),
         function=remote_call,
-        availability=None,
-        format_result=_format_tool_result,
-        format_error=_format_tool_failure,
-        handling="auto",
+        availability=_available,
+        result_formatting=_format_tool_result,
+        error_formatting=_format_tool_failure,
+        handling="response",
         meta=Meta(
             {
                 "mcp_server": source,
@@ -613,6 +508,13 @@ def _convert_tool(
     )
 
 
+def _available(
+    tools_turn: int,
+    specification: ModelToolSpecification,
+) -> bool:
+    return True
+
+
 def _format_tool_result(
     result: Any,
 ) -> MultimodalContent:
@@ -644,16 +546,15 @@ class MCPClients:
         *clients: MCPClient,
     ) -> None:
         self._clients: Mapping[str, MCPClient] = {c.identifier: c for c in [client, *clients]}
-        self._resources: Mapping[str, Resources]
-        self._prompts: Mapping[str, Prompts]
-        self._tools: Mapping[str, Tools]
+        self._resources: Mapping[str, ResourcesRepository]
+        self._tools: Mapping[str, ToolsProvider]
 
     async def resources_list(
         self,
         *,
         mcp_server: str | None = None,
         **extra: Any,
-    ) -> Sequence[ResourceDeclaration]:
+    ) -> Sequence[ResourceReference]:
         if mcp_server is None:
             return tuple(
                 chain.from_iterable(
@@ -673,9 +574,9 @@ class MCPClients:
         self,
         uri: str,
         **extra: Any,
-    ) -> Resource | None:
+    ) -> Collection[ResourceReference] | ResourceContent | None:
         if client_identifier := self._client_identifier_for_uri(uri):
-            return await self._resources[client_identifier].fetch(uri)
+            return await self._resources[client_identifier].fetching(uri)
 
         else:
             ctx.log_warning(f"Requested resource ({uri}) from unknown source")
@@ -683,10 +584,18 @@ class MCPClients:
 
     async def resource_upload(
         self,
-        resource: Resource,
+        uri: str,
+        content: ResourceContent,
+        **extra: Any,
+    ) -> Meta:
+        raise NotImplementedError("Resource uploading is not supported by MCP servers")
+
+    async def resource_delete(
+        self,
+        uri: str,
         **extra: Any,
     ) -> None:
-        raise NotImplementedError("Resource uploading is not supported by MCP servers")
+        raise NotImplementedError("Resource deleting is not supported by MCP servers")
 
     def _client_identifier_for_uri(
         self,
@@ -711,57 +620,6 @@ class MCPClients:
             case _:
                 return None
 
-    async def prompts_list(
-        self,
-        *,
-        mcp_server: str | None = None,
-        **extra: Any,
-    ) -> Sequence[PromptDeclaration]:
-        if mcp_server is None:
-            return tuple(
-                chain.from_iterable(
-                    await gather(*[client.fetch_list(**extra) for client in self._prompts.values()])
-                )
-            )
-
-        elif prompts := self._prompts.get(mcp_server):
-            return await prompts.fetch_list(**extra)
-
-        else:
-            return ()
-
-    async def prompt_fetch(
-        self,
-        name: str,
-        *,
-        arguments: Mapping[str, str] | None,
-        **extra: Any,
-    ) -> Prompt | None:
-        if client_identifier := self._client_identifier_for_prompt_name(name):
-            return await self._prompts[client_identifier].fetch(
-                name,
-                arguments=arguments,
-            )
-
-        else:
-            ctx.log_warning(f"Requested prompt ({name}) from unknown source")
-            return None
-
-    def _client_identifier_for_prompt_name(
-        self,
-        name: str,
-        /,
-    ) -> str | None:
-        if not name:
-            return None
-
-        match name.split("|", 1):
-            case [identifier, _]:
-                return identifier
-
-            case _:
-                return None
-
     async def tools_fetch(
         self,
         *,
@@ -771,18 +629,18 @@ class MCPClients:
         if mcp_server is None:
             return tuple(
                 chain.from_iterable(
-                    await gather(*[client.fetch(**extra) for client in self._tools.values()])
+                    await gather(*[client.loading(**extra) for client in self._tools.values()])
                 )
             )
 
         elif tools := self._tools.get(mcp_server):
-            return await tools.fetch(**extra)
+            return await tools.loading(**extra)
 
         else:
             return ()
 
-    async def __aenter__(self) -> Sequence[Resources | Prompts | Tools]:
-        features: Sequence[Sequence[Resources | Prompts | Tools]] = await gather(
+    async def __aenter__(self) -> Sequence[ResourcesRepository | ToolsProvider]:
+        features: Sequence[Sequence[ResourcesRepository | ToolsProvider]] = await gather(
             *[client.__aenter__() for client in self._clients.values()]
         )
 
@@ -792,40 +650,28 @@ class MCPClients:
 
         for states in features:
             for state in states:
-                match state:
-                    case Resources() as resources:
-                        self._resources[cast(str, resources.meta["mcp_server"])] = resources
+                if isinstance(state, ResourcesRepository):
+                    self._resources[cast(str, state.meta["mcp_server"])] = state
 
-                    case Prompts() as prompts:
-                        self._prompts[cast(str, prompts.meta["mcp_server"])] = prompts
+                if isinstance(state, ToolsProvider):
+                    self._tools[cast(str, state.meta["mcp_server"])] = state
 
-                    case Tools() as tools:
-                        self._tools[cast(str, tools.meta["mcp_server"])] = tools
-
-        inherited_features: list[Resources | Prompts | Tools] = []
+        inherited_features: list[ResourcesRepository | ToolsProvider] = []
         if self._resources:
             inherited_features.append(
-                Resources(
+                ResourcesRepository(
                     list_fetching=self.resources_list,
                     fetching=self.resource_fetch,
                     uploading=self.resource_upload,
-                    meta=Meta({"mcp_server": "mcp_aggregate"}),
-                )
-            )
-
-        if self._prompts:
-            inherited_features.append(
-                Prompts(
-                    list_fetching=self.prompts_list,
-                    fetching=self.prompt_fetch,
+                    deleting=self.resource_delete,
                     meta=Meta({"mcp_server": "mcp_aggregate"}),
                 )
             )
 
         if self._tools:
             inherited_features.append(
-                Tools(
-                    fetching=self.tools_fetch,
+                ToolsProvider(
+                    loading=self.tools_fetch,
                     meta=Meta({"mcp_server": "mcp_aggregate"}),
                 )
             )
