@@ -1,5 +1,5 @@
-from base64 import urlsafe_b64encode
-from collections.abc import AsyncGenerator, Iterable, Mapping, Sequence
+from base64 import urlsafe_b64decode, urlsafe_b64encode
+from collections.abc import AsyncGenerator, Collection, Iterable, Mapping, Sequence
 from contextlib import asynccontextmanager
 from typing import Any, final
 from uuid import uuid4
@@ -9,11 +9,9 @@ from haiway import Disposable, Disposables, State, as_dict, ctx
 from mcp.server import NotificationOptions, Server
 from mcp.server.lowlevel.helper_types import ReadResourceContents
 from mcp.shared.message import SessionMessage
-from mcp.types import EmbeddedResource, GetPromptResult
+from mcp.types import AudioContent as MCPAudioContent
+from mcp.types import BlobResourceContents, EmbeddedResource
 from mcp.types import ImageContent as MCPImageContent
-from mcp.types import Prompt as MCPPrompt
-from mcp.types import PromptArgument as MCPPromptArgument
-from mcp.types import PromptMessage as MCPPromptMessage
 from mcp.types import Resource as MCPResource
 from mcp.types import ResourceTemplate as MCPResourceTemplate
 from mcp.types import TextContent as MCPTextContent
@@ -21,11 +19,9 @@ from mcp.types import Tool as MCPTool
 from pydantic import AnyUrl
 from starlette.types import ASGIApp
 
-from draive.lmm import LMMCompletion, LMMContextElement, LMMInput
-from draive.multimodal import MediaData, MediaReference, MultimodalContent, TextContent
-from draive.prompts import Prompt, PromptTemplate
-from draive.resources import Resource, ResourceContent, ResourceTemplate
-from draive.tools import Tool, Toolbox
+from draive.models import Tool, Toolbox
+from draive.multimodal import ArtifactContent, MultimodalContent, TextContent
+from draive.resources import Resource, ResourceContent, ResourceReference, ResourceTemplate
 
 __all__ = ("MCPServer",)
 
@@ -41,20 +37,18 @@ class MCPServer:
         version: str | None = None,
         instructions: str | None = None,
         resources: Iterable[ResourceTemplate | Resource] | None = None,
-        prompts: Iterable[PromptTemplate[Any] | Prompt] | None = None,
         tools: Toolbox | Iterable[Tool] | None = None,
-        disposables: Disposables | Iterable[Disposable] | None = None,
+        disposables: Disposables | Collection[Disposable] | None = None,
     ) -> None:
         disposable: Disposables
-        match disposables:
-            case None:
-                disposable = Disposables()
+        if disposables is None:
+            disposable = Disposables(())
 
-            case Disposables() as prepared:
-                disposable = prepared
+        elif isinstance(disposables, Disposables):
+            disposable = disposables
 
-            case other:
-                disposable = Disposables(*other)
+        else:
+            disposable = Disposables(disposables)
 
         @asynccontextmanager
         async def lifspan(server: Server) -> AsyncGenerator[Iterable[State]]:
@@ -75,16 +69,13 @@ class MCPServer:
         if resources is not None:
             self._expose_resources(resources)
 
-        if prompts is not None:
-            self._expose_prompts(prompts)
-
         if tools is not None:
             self._expose_tools(tools)
 
     async def run_stdio(
         self,
         notification_options: NotificationOptions | None = None,
-        experimental_capabilities: dict[str, dict[str, Any]] | None = None,
+        experimental_capabilities: Mapping[str, dict[str, Any]] | None = None,
     ) -> None:
         from mcp.server.stdio import stdio_server
 
@@ -93,13 +84,13 @@ class MCPServer:
                 read_stream=streams[0],
                 write_stream=streams[1],
                 notification_options=notification_options,
-                experimental_capabilities=experimental_capabilities,
+                experimental_capabilities=as_dict(experimental_capabilities),
             )
 
     def prepare_asgi(
         self,
         notification_options: NotificationOptions | None = None,
-        experimental_capabilities: dict[str, dict[str, Any]] | None = None,
+        experimental_capabilities: Mapping[str, dict[str, Any]] | None = None,
     ) -> ASGIApp:
         from mcp.server.sse import SseServerTransport
         from starlette.applications import Starlette
@@ -113,7 +104,7 @@ class MCPServer:
                     read_stream=streams[0],
                     write_stream=streams[1],
                     notification_options=notification_options,
-                    experimental_capabilities=experimental_capabilities,
+                    experimental_capabilities=as_dict(experimental_capabilities),
                 )
 
         return Starlette(
@@ -160,8 +151,8 @@ class MCPServer:
                                 MCPResource(
                                     uri=AnyUrl(resource.uri),
                                     mimeType=content.mime_type,
-                                    name=resource.name,
-                                    description=resource.description,
+                                    name=resource.meta.name or resource.uri,
+                                    description=resource.meta.description,
                                 )
                             )
 
@@ -174,10 +165,11 @@ class MCPServer:
                     if resource.arguments:
                         resource_template_declarations.append(
                             MCPResourceTemplate(
-                                uriTemplate=resource.declaration.uri_template,
+                                uriTemplate=resource.declaration.template_uri,
                                 mimeType=resource.declaration.mime_type,
-                                name=resource.declaration.name,
-                                description=resource.declaration.description,
+                                name=resource.declaration.meta.name
+                                or resource.declaration.template_uri,
+                                description=resource.declaration.meta.description,
                             )
                         )
                         # TODO: we might need to sort based on template uri matching priorities
@@ -186,14 +178,15 @@ class MCPServer:
                     else:
                         resource_declarations.append(
                             MCPResource(
-                                uri=AnyUrl(resource.declaration.uri_template),
+                                uri=AnyUrl(resource.declaration.template_uri),
                                 mimeType=resource.declaration.mime_type,
-                                name=resource.declaration.name,
-                                description=resource.declaration.description,
+                                name=resource.declaration.meta.name
+                                or resource.declaration.template_uri,
+                                description=resource.declaration.meta.description,
                             )
                         )
                         # we treat it as a regular resource if template has no arguments
-                        available_resources[resource.declaration.uri_template] = resource
+                        available_resources[resource.declaration.template_uri] = resource
 
         if resource_declarations:
 
@@ -243,80 +236,6 @@ class MCPServer:
 
                 return _resource_content(resource)
 
-    def _expose_prompts(
-        self,
-        prompts: Iterable[PromptTemplate[Any] | Prompt],
-        /,
-    ) -> None:
-        prompt_declarations: list[MCPPrompt] = []
-        available_prompts: dict[str, PromptTemplate[Any] | Prompt] = {}
-        for prompt in prompts:
-            match prompt:
-                case Prompt():
-                    prompt_declarations.append(
-                        MCPPrompt(
-                            name=prompt.name,
-                            arguments=None,
-                            description=prompt.description,
-                        )
-                    )
-                    available_prompts[prompt.name] = prompt
-
-                case PromptTemplate():
-                    prompt_declarations.append(
-                        MCPPrompt(
-                            name=prompt.declaration.name,
-                            arguments=[
-                                MCPPromptArgument(
-                                    name=argument.name,
-                                    description=argument.specification.get("description"),
-                                    required=argument.required,
-                                )
-                                for argument in prompt.declaration.arguments
-                            ],
-                            description=prompt.declaration.description,
-                        )
-                    )
-                    available_prompts[prompt.declaration.name] = prompt
-
-        @self._server.list_prompts()
-        async def list_prompts() -> list[MCPPrompt]:  # pyright: ignore[reportUnusedFunction]
-            async with ctx.scope(
-                "list_prompts",
-                *self._server.request_context.lifespan_context,
-            ):
-                return prompt_declarations
-
-        @self._server.get_prompt()
-        async def get_prompt(  # pyright: ignore[reportUnusedFunction]
-            name: str,
-            arguments: Mapping[str, Any] | None,
-        ) -> GetPromptResult:
-            async with ctx.scope(
-                "get_prompt",
-                *self._server.request_context.lifespan_context,
-            ):
-                match available_prompts.get(name):
-                    case None:
-                        raise ValueError(f"Prompt '{name}' is not defined")
-
-                    case Prompt() as direct_prompt:
-                        return GetPromptResult(
-                            description=direct_prompt.description,
-                            messages=[
-                                _convert_message(element) for element in direct_prompt.content
-                            ],
-                        )
-
-                    case PromptTemplate() as dynamic_prompt:
-                        resolved_prompt: Prompt = await dynamic_prompt.resolve(arguments or {})
-                        return GetPromptResult(
-                            description=resolved_prompt.description,
-                            messages=[
-                                _convert_message(element) for element in resolved_prompt.content
-                            ],
-                        )
-
     def _expose_tools(
         self,
         tools: Toolbox | Iterable[Tool],
@@ -338,9 +257,9 @@ class MCPServer:
             ):
                 return [
                     MCPTool(
-                        name=tool["name"],
-                        description=tool["description"],
-                        inputSchema=as_dict(tool["parameters"]) or {},
+                        name=tool.name,
+                        description=tool.description,
+                        inputSchema=as_dict(tool.parameters) or {},
                     )
                     for tool in (
                         tool.specification for tool in toolbox.tools.values() if tool.available
@@ -351,7 +270,7 @@ class MCPServer:
         async def call_tool(  # pyright: ignore[reportUnusedFunction]
             name: str,
             arguments: Mapping[str, Any],
-        ) -> Sequence[MCPTextContent | MCPImageContent | EmbeddedResource]:
+        ) -> Sequence[MCPTextContent | MCPImageContent | MCPAudioContent | EmbeddedResource]:
             async with ctx.scope(
                 "call_tool",
                 *self._server.request_context.lifespan_context,
@@ -373,78 +292,25 @@ def _resource_content(
             match content.mime_type:
                 case "text/plain" | "application/json":
                     yield ReadResourceContents(
-                        content=content.blob.decode(),
+                        content=urlsafe_b64decode(content.data).decode(),
                         mime_type=content.mime_type,
                     )
 
                 case _:
                     yield ReadResourceContents(
-                        content=content.blob,
+                        content=urlsafe_b64decode(content.data),
                         mime_type=content.mime_type,
                     )
 
-        case [*contents]:  # is it intended use of nested resource contents?
-            for content in contents:
-                yield from _resource_content(content)
-
-
-def _convert_message(
-    element: LMMContextElement,
-    /,
-) -> MCPPromptMessage:
-    match element:
-        case LMMInput():
-            match _convert_multimodal_content(element.content):
-                case [content]:
-                    return MCPPromptMessage(
-                        role="user",
-                        content=content,
-                    )
-                case []:
-                    return MCPPromptMessage(
-                        role="user",
-                        # convert empty content to empty text to prevent errors
-                        content=MCPTextContent(
-                            type="text",
-                            text="",
-                        ),
-                    )
-
-                case [*_]:
-                    raise NotImplementedError(
-                        f"Unsuppoprted MCP prompt element content: {element.content}!"
-                    )
-
-        case LMMCompletion():
-            match _convert_multimodal_content(element.content):
-                case [content]:
-                    return MCPPromptMessage(
-                        role="assistant",
-                        content=content,
-                    )
-                case []:
-                    return MCPPromptMessage(
-                        role="assistant",
-                        # convert empty content to empty text to prevent errors
-                        content=MCPTextContent(
-                            type="text",
-                            text="",
-                        ),
-                    )
-
-                case [*_]:
-                    raise NotImplementedError(
-                        f"Unsuppoprted MCP prompt element content: {element.content}!"
-                    )
-
-        case _:
-            raise NotImplementedError(f"Unsuppoprted MCP prompt context element: {type(element)}!")
+        case [*_]:
+            # Multi-content resources (lists of references) are not supported for server reads
+            raise NotImplementedError("Multi-content resources are not supported yet")
 
 
 def _convert_multimodal_content(
     content: MultimodalContent,
-) -> Sequence[MCPTextContent | MCPImageContent | EmbeddedResource]:
-    converted: list[MCPTextContent | MCPImageContent | EmbeddedResource] = []
+) -> Sequence[MCPTextContent | MCPImageContent | MCPAudioContent | EmbeddedResource]:
+    converted: list[MCPTextContent | MCPImageContent | MCPAudioContent | EmbeddedResource] = []
     for part in content.parts:
         match part:
             case TextContent() as text:
@@ -455,32 +321,93 @@ def _convert_multimodal_content(
                     )
                 )
 
-            case MediaData() as media_data:
-                if media_data.kind != "image":
-                    raise NotImplementedError(
-                        f"Media support for {media_data.media} is not implemented"
+            case ResourceContent() as data:
+                mime: str = data.mime_type
+                if mime.startswith("image"):
+                    converted.append(
+                        MCPImageContent(
+                            type="image",
+                            data=data.data,
+                            mimeType=mime,
+                        )
                     )
 
-                converted.append(
-                    MCPImageContent(
-                        type="image",
-                        data=urlsafe_b64encode(media_data.data).decode(),
-                        mimeType=media_data.media,
+                elif mime.startswith("audio"):
+                    converted.append(
+                        MCPAudioContent(
+                            type="audio",
+                            data=data.data,
+                            mimeType=mime,
+                        )
                     )
+
+                elif mime == "text/plain":
+                    converted.append(
+                        MCPTextContent(
+                            type="text",
+                            text=urlsafe_b64decode(data.data).decode(),
+                        )
+                    )
+
+                elif mime == "application/json":
+                    encoded: str = data.data
+                    # Provide a data URI to satisfy required AnyUrl
+                    uri = AnyUrl(f"data:{mime};base64,{encoded}")
+                    converted.append(
+                        EmbeddedResource(
+                            type="resource",
+                            resource=BlobResourceContents(
+                                uri=uri,
+                                mimeType=mime,
+                                blob=encoded,
+                            ),
+                        )
+                    )
+
+                else:
+                    # Unknown blob types: embed as a resource blob
+                    encoded: str = data.data
+                    uri = AnyUrl(f"data:{mime};base64,{encoded}")
+                    converted.append(
+                        EmbeddedResource(
+                            type="resource",
+                            resource=BlobResourceContents(
+                                uri=uri,
+                                mimeType=mime,
+                                blob=encoded,
+                            ),
+                        )
+                    )
+
+            case ResourceReference():
+                # We don't return links yet; ask callers to provide content
+                # we could try to resolve those contextually using ResourceRepository
+                raise NotImplementedError(
+                    "MCP resource links are not supported yet; provide content blobs instead"
                 )
 
-            case MediaReference():
-                # TODO: download images on the fly?
-                raise NotImplementedError(
-                    "Media reference support is not implemented, please use data blobs instead"
+            case ArtifactContent() as artifact:
+                encoded: str = urlsafe_b64encode(artifact.artifact.to_json().encode()).decode()
+                uri = AnyUrl(f"data:application/json;base64,{encoded}")
+                converted.append(
+                    EmbeddedResource(
+                        type="resource",
+                        resource=BlobResourceContents(
+                            uri=uri,
+                            mimeType="application/json",
+                            blob=encoded,
+                        ),
+                    )
                 )
 
             case other:
+                # Fallback: serialize unknown parts to str
                 converted.append(
                     MCPTextContent(
                         type="text",
-                        text=other.to_json(),
+                        text=other.to_str(),
                     )
                 )
+                raise ValueError(f"Unknown content type: {type(other)}")
 
     return converted
