@@ -1,22 +1,25 @@
 from collections.abc import Collection, Iterable
 from types import TracebackType
-from typing import Literal, final
+from typing import Any, final
 
 from haiway import State
 
+from draive.embedding import TextEmbedding
+from draive.guardrails import GuardrailsModeration
 from draive.mistral.api import MistralAPI
+from draive.mistral.completions import MistralCompletions
 from draive.mistral.embedding import MistralEmbedding
-from draive.mistral.lmm_generation import MistralLMMGeneration
-from draive.mistral.tokenization import MistralTokenization
+from draive.mistral.moderation import MistralContentModeration
+from draive.models import GenerativeModel
 
 __all__ = ("Mistral",)
 
 
 @final
 class Mistral(
-    MistralLMMGeneration,
+    MistralCompletions,
     MistralEmbedding,
-    MistralTokenization,
+    MistralContentModeration,
     MistralAPI,
 ):
     """
@@ -29,28 +32,37 @@ class Mistral(
         self,
         server_url: str | None = None,
         api_key: str | None = None,
-        timeout: float = 60.0,
-        features: Collection[Literal["lmm", "text_embedding"]] | None = None,
+        timeout: float | None = None,
+        features: Collection[type[GenerativeModel | TextEmbedding | GuardrailsModeration]]
+        | None = None,
+        **extra: Any,
     ) -> None:
         super().__init__(
             server_url=server_url,
             api_key=api_key,
             timeout=timeout,
+            **extra,
         )
 
-        self._features: frozenset[Literal["lmm", "text_embedding"]] = (
-            frozenset(features) if features is not None else frozenset(("lmm", "text_embedding"))
+        self._features: frozenset[type[State]] = (
+            frozenset(features)
+            if features is not None
+            else frozenset((GenerativeModel, TextEmbedding, GuardrailsModeration))
         )
 
     async def __aenter__(self) -> Iterable[State]:
         await self._initialize_client()
         state: list[State] = []
-        if "lmm" in self._features:
-            state.append(self.lmm())
 
-        if "text_embedding" in self._features:
+        if GenerativeModel in self._features:
+            state.append(self.generative_model())
+
+        if TextEmbedding in self._features:
             state.append(self.text_embedding())
 
+        if GuardrailsModeration in self._features:
+            state.append(self.content_guardrails())
+
         return state
 
     async def __aexit__(
