@@ -1,10 +1,8 @@
 from draive.ollama.client import Ollama
 from draive.ollama.config import OllamaChatConfig, OllamaEmbeddingConfig
-from draive.ollama.types import OllamaException
 
 __all__ = (
     "Ollama",
     "OllamaChatConfig",
     "OllamaEmbeddingConfig",
-    "OllamaException",
 )
