@@ -1,8 +1,6 @@
 from collections.abc import Sequence
 
-from haiway import MISSING, Missing
-
-from draive.configuration import Config
+from haiway import MISSING, Configuration, Missing
 
 __all__ = (
     "OllamaChatConfig",
@@ -10,18 +8,17 @@ __all__ = (
 )
 
 
-class OllamaChatConfig(Config):
+class OllamaChatConfig(Configuration):
     model: str
     temperature: float = 1.0
     top_k: int | Missing = MISSING
     top_p: float | Missing = MISSING
     seed: int | None | Missing = MISSING
-    max_tokens: int | Missing = MISSING
-    timeout: float | Missing = MISSING
+    max_output_tokens: int | Missing = MISSING
     stop_sequences: Sequence[str] | Missing = MISSING
 
 
-class OllamaEmbeddingConfig(Config):
+class OllamaEmbeddingConfig(Configuration):
     model: str
     concurrent: bool = False
     batch_size: int = 32
