@@ -5,7 +5,7 @@ from haiway import MISSING, Missing
 __all__ = ("unwrap_missing",)
 
 
-def unwrap_missing[Value, Default](
+def unwrap_missing[Value](
     value: Value | Missing,
     /,
 ) -> Value | None:
