@@ -1,19 +1,17 @@
 from draive.openai.client import OpenAI
 from draive.openai.config import (
-    OpenAIChatConfig,
     OpenAIEmbeddingConfig,
     OpenAIImageGenerationConfig,
     OpenAIModerationConfig,
     OpenAIRealtimeConfig,
+    OpenAIResponsesConfig,
 )
-from draive.openai.types import OpenAIException
 
 __all__ = (
     "OpenAI",
-    "OpenAIChatConfig",
     "OpenAIEmbeddingConfig",
-    "OpenAIException",
     "OpenAIImageGenerationConfig",
     "OpenAIModerationConfig",
     "OpenAIRealtimeConfig",
+    "OpenAIResponsesConfig",
 )
