@@ -5,17 +5,11 @@ from openai.types.image import Image
 from openai.types.images_response import ImagesResponse
 
 from draive.generation import ImageGeneration
-from draive.instructions import Instruction
-from draive.multimodal import (
-    MediaContent,
-    MediaData,
-    MediaReference,
-    Multimodal,
-)
+from draive.models import InstructionsRepository, ResolveableInstructions
+from draive.multimodal import MultimodalContent
 from draive.openai.api import OpenAIAPI
 from draive.openai.config import OpenAIImageGenerationConfig
-from draive.openai.types import OpenAIException
-from draive.openai.utils import unwrap_missing
+from draive.resources import ResourceContent, ResourceReference
 
 __all__ = ("OpenAIImageGeneration",)
 
@@ -27,41 +21,40 @@ class OpenAIImageGeneration(OpenAIAPI):
     async def generate_image(
         self,
         *,
-        instruction: Instruction | str,
-        input: Multimodal | None,  # noqa: A002
+        instructions: ResolveableInstructions,
+        input: MultimodalContent,  # noqa: A002
         config: OpenAIImageGenerationConfig | None = None,
         **extra: Any,
-    ) -> MediaContent:
+    ) -> ResourceContent | ResourceReference:
         generation_config: OpenAIImageGenerationConfig = config or ctx.state(
             OpenAIImageGenerationConfig
         )
-        async with ctx.scope("generate_image", generation_config):
+        async with ctx.scope("generate_image"):
             response: ImagesResponse = await self._client.images.generate(
                 model=generation_config.model,
                 n=1,
-                prompt=Instruction.formatted(instruction),
+                prompt=await InstructionsRepository.resolve(instructions),
                 quality=generation_config.quality,
                 size=generation_config.size,
                 style=generation_config.style,
-                timeout=unwrap_missing(generation_config.timeout),
                 response_format=generation_config.result,
             )
 
             if response.data is None:
-                raise OpenAIException("Invalid OpenAI response - missing image content")
+                raise ValueError("Invalid OpenAI response - missing image content")
 
             image: Image = response.data[0]
             if url := image.url:
-                return MediaReference.of(
+                return ResourceReference.of(
                     url,
-                    media="image/png",
+                    mime_type="image/png",
                 )
 
             elif b64data := image.b64_json:
-                return MediaData.of(
+                return ResourceContent.of(
                     b64data,
-                    media="image/png",
+                    mime_type="image/png",
                 )
 
             else:
-                raise OpenAIException("Invalid OpenAI response - missing image content")
+                raise ValueError("Invalid OpenAI response - missing image content")
