@@ -1,16 +1,17 @@
 from typing import cast
 
 from haiway import MISSING, Missing
-from openai import NOT_GIVEN, NotGiven
+from openai import Omit, omit
 
 __all__ = ("unwrap_missing",)
 
 
-def unwrap_missing[Value, Default](
+def unwrap_missing[Value](
     value: Value | Missing,
     /,
-) -> Value | NotGiven:
+) -> Value | Omit:
     if value is MISSING:
-        return NOT_GIVEN
+        return omit
+
     else:
         return cast(Value, value)
