@@ -2,89 +2,43 @@ from collections.abc import Callable, Iterable, Mapping
 from inspect import Parameter as InspectParameter
 from inspect import _empty as INSPECT_EMPTY  # pyright: ignore[reportPrivateUsage]
 from inspect import signature
-from types import EllipsisType
-from typing import Any, ClassVar, cast, final, get_type_hints, overload
+from typing import Any, cast, final, get_type_hints
 
-from haiway import MISSING, DefaultValue, Missing
-from haiway.state import AttributeAnnotation
-from haiway.state.attributes import resolve_attribute_annotation
+from haiway import (
+    MISSING,
+    AttributeAnnotation,
+    DefaultValue,
+    Missing,
+    TypeSpecification,
+    ValidationContext,
+    not_missing,
+)
+from haiway.attributes import Attribute
+from haiway.attributes.annotations import resolve_attribute
+from haiway.attributes.specification import type_specification
 from haiway.utils import mimic_function
 
-from draive.parameters.parameter import Parameter
-from draive.parameters.specification import ParameterSpecification
-from draive.parameters.types import ParameterValidation, ParameterVerification
-from draive.parameters.validation import ParameterValidationContext
-
 __all__ = (
     "Argument",
     "ParametrizedFunction",
 )
 
 
-@overload
-def Argument[Value](
-    *,
-    aliased: str | None = None,
-    description: str | Missing = MISSING,
-    default: Value | Missing = MISSING,
-    validator: ParameterValidation[Value] | Missing = MISSING,
-    specification: ParameterSpecification | Missing = MISSING,
-) -> Value: ...
-
-
-@overload
-def Argument[Value](
-    *,
-    aliased: str | None = None,
-    description: str | Missing = MISSING,
-    default_factory: Callable[[], Value] | Missing = MISSING,
-    validator: ParameterValidation[Value] | Missing = MISSING,
-    specification: ParameterSpecification | Missing = MISSING,
-) -> Value: ...
-
-
-@overload
-def Argument[Value](
-    *,
-    aliased: str | None = None,
-    description: str | Missing = MISSING,
-    default: Value | Missing = MISSING,
-    verifier: ParameterVerification[Value] | Missing = MISSING,
-    specification: ParameterSpecification | Missing = MISSING,
-) -> Value: ...
-
-
-@overload
-def Argument[Value](
-    *,
-    aliased: str | None = None,
-    description: str | Missing = MISSING,
-    default_factory: Callable[[], Value] | Missing = MISSING,
-    verifier: ParameterVerification[Value] | Missing = MISSING,
-    specification: ParameterSpecification | Missing = MISSING,
-) -> Value: ...
-
-
 def Argument[Value](
     *,
     aliased: str | None = None,
     description: str | Missing = MISSING,
     default: Value | Missing = MISSING,
     default_factory: Callable[[], Value] | Missing = MISSING,
-    validator: ParameterValidation[Value] | Missing = MISSING,
-    verifier: ParameterVerification[Value] | Missing = MISSING,
-    specification: ParameterSpecification | Missing = MISSING,
+    default_env: str | Missing = MISSING,
+    specification: TypeSpecification | Missing = MISSING,
 ) -> Value:  # it is actually a FunctionArgument, but type checker has to be fooled
     assert (  # nosec: B101
-        default is MISSING or default_factory is MISSING
-    ), "Can't specify both default value and factory"
+        default is MISSING or default_factory is MISSING or default_env is MISSING
+    ), "Can't specify default value, factory and env"
     assert (  # nosec: B101
         description is MISSING or specification is MISSING
     ), "Can't specify both description and specification"
-    assert (  # nosec: B101
-        validator is MISSING or verifier is MISSING
-    ), "Can't specify both validator and verifier"
-
     return cast(
         Value,
         FunctionArgument(
@@ -92,8 +46,7 @@ def Argument[Value](
             description=description,
             default=default,
             default_factory=default_factory,
-            validator=validator,
-            verifier=verifier,
+            default_env=default_env,
             specification=specification,
         ),
     )
@@ -107,22 +60,18 @@ class FunctionArgument:
         description: str | Missing,
         default: Any | Missing,
         default_factory: Callable[[], Any] | Missing,
-        validator: ParameterValidation[Any] | Missing,
-        verifier: ParameterVerification[Any] | Missing,
-        specification: ParameterSpecification | Missing,
+        default_env: str | Missing,
+        specification: TypeSpecification | Missing,
     ) -> None:
         self.aliased: str | None = aliased
         self.description: str | Missing = description
         self.default: Any | Missing = default
         self.default_factory: Callable[[], Any] | Missing = default_factory
-        self.validator: ParameterValidation[Any] | Missing = validator
-        self.verifier: ParameterVerification[Any] | Missing = verifier
-        self.specification: ParameterSpecification | Missing = specification
+        self.default_env: str | Missing = default_env
+        self.specification: TypeSpecification | Missing = specification
 
 
 class ParametrizedFunction[**Args, Result]:
-    __IMMUTABLE__: ClassVar[EllipsisType] = ...
-
     __slots__ = (
         "__defaults__",
         "__doc__",
@@ -159,19 +108,22 @@ class ParametrizedFunction[**Args, Result]:
             "_name",
             function.__name__,
         )
-        self._parameters: dict[str, Parameter[Any]]
+        self._parameters: dict[str, Attribute[Any]]
         object.__setattr__(
             self,
             "_parameters",
             {},
         )
-        self._variadic_keyword_parameters: Parameter[Any] | None
+        self._variadic_keyword_parameters: Attribute[Any] | None
         object.__setattr__(
             self,
             "_variadic_keyword_parameters",
             None,
         )
-        type_hints: Mapping[str, Any] = get_type_hints(function)
+        type_hints: Mapping[str, Any] = get_type_hints(
+            function,
+            include_extras=True,
+        )
         for parameter in signature(function).parameters.values():
             match parameter.kind:
                 case InspectParameter.POSITIONAL_ONLY | InspectParameter.VAR_POSITIONAL:
@@ -199,7 +151,7 @@ class ParametrizedFunction[**Args, Result]:
         mimic_function(function, within=self)
 
     @property
-    def arguments(self) -> Iterable[Parameter[Any]]:
+    def arguments(self) -> Iterable[Attribute[Any]]:
         return self._parameters.values()
 
     def validate_arguments(
@@ -207,29 +159,24 @@ class ParametrizedFunction[**Args, Result]:
         **kwargs: Any,
     ) -> dict[str, Any]:
         # TODO: add support for positional arguments
-        with ParameterValidationContext().scope(self.__class__.__qualname__) as context:
-            validated: dict[str, Any] = {}
-            if self._variadic_keyword_parameters is None:
-                for parameter in self._parameters.values():
-                    validated[parameter.name] = parameter.validated(
-                        parameter.find(kwargs),
-                        context=context,
-                    )
+        validated: dict[str, Any] = {}
+        if self._variadic_keyword_parameters is None:
+            for parameter in self._parameters.values():
+                with ValidationContext.scope(f".{parameter.name}"):
+                    validated[parameter.name] = parameter.validate_from(kwargs)
 
-            else:
-                for parameter in self._parameters.values():
-                    validated[parameter.name] = parameter.validated(
-                        parameter.pick(kwargs),
-                        context=context,
-                    )
+        else:
+            for parameter in self._parameters.values():
+                with ValidationContext.scope(f".{parameter.name}"):
+                    validated[parameter.name] = parameter.validate_from(kwargs)
 
-                for key, value in kwargs.items():
-                    validated[key] = self._variadic_keyword_parameters.validated(
+            for key, value in kwargs.items():
+                with ValidationContext.scope(f".{key}"):
+                    validated[key] = self._variadic_keyword_parameters.validate(
                         value,
-                        context=context,
                     )
 
-            return validated
+        return validated
 
     def __call__(
         self,
@@ -265,65 +212,73 @@ def _resolve_argument(
     *,
     module: str,
     type_hint: Any,
-) -> Parameter[Any]:
+) -> Attribute[Any]:
     if parameter.annotation is INSPECT_EMPTY or type_hint is None:
-        raise TypeError(
-            "Untyped argument %s",
-            parameter.name,
-        )
+        raise TypeError(f"Untyped argument {parameter.name}")
 
-    attribute: AttributeAnnotation = resolve_attribute_annotation(
+    attribute: AttributeAnnotation = resolve_attribute(
         type_hint,
         module=module,
-        type_parameters={},
-        self_annotation=None,
+        resolved_parameters={},
         recursion_guard={},
     )
 
     match parameter.default:
         case FunctionArgument() as argument:
-            return Parameter[Any].of(
-                attribute,
+            alias: str | None = (
+                argument.aliased
+                if argument.aliased is not None
+                else attribute.alias
+            )
+            specification: TypeSpecification | None = None
+
+            if not_missing(argument.specification):
+                specification = argument.specification
+
+            else:
+                description: str | None = None
+                if not_missing(argument.description):
+                    description = argument.description
+
+                else:
+                    description = attribute.description
+
+                specification = type_specification(
+                    attribute,
+                    description=description,
+                )
+
+            return Attribute(
                 name=parameter.name,
-                alias=argument.aliased,
-                description=argument.description,
+                annotation=attribute,
+                alias=alias,
                 default=DefaultValue(
                     argument.default,
                     factory=argument.default_factory,
+                    env=argument.default_env,
                 ),
-                validator=argument.validator,
-                verifier=argument.verifier,
-                converter=MISSING,
-                specification=argument.specification,
-                required=attribute.required
-                and argument.default is MISSING
-                and argument.default_factory is MISSING,
+                specification=specification,
+                required=argument.default is MISSING
+                and argument.default_factory is MISSING
+                and argument.default_env is MISSING,
             )
 
-        case DefaultValue() as default:  # pyright: ignore[reportUnknownVariableType]
-            return Parameter[Any].of(
-                attribute,
+        case DefaultValue() as default:
+            return Attribute(
                 name=parameter.name,
-                alias=None,
-                description=MISSING,
-                default=default,  # pyright: ignore[reportUnknownArgumentType]
-                validator=MISSING,
-                verifier=MISSING,
-                converter=MISSING,
-                specification=MISSING,
+                annotation=attribute,
+                alias=attribute.alias,
+                default=default,
+                specification=type_specification(attribute, None),
                 required=False,
             )
 
         case value:
-            return Parameter[Any].of(
-                attribute,
+            return Attribute(
                 name=parameter.name,
-                alias=None,
-                description=MISSING,
+                annotation=attribute,
+                alias=attribute.alias,
                 default=DefaultValue(MISSING if value is INSPECT_EMPTY else value),
-                validator=MISSING,
-                verifier=MISSING,
-                converter=MISSING,
-                specification=MISSING,
-                required=attribute.required and value is INSPECT_EMPTY,
+                specification=type_specification(attribute, None),
+                required=value is INSPECT_EMPTY and attribute.required,
             )
