@@ -3,11 +3,9 @@ from draive.vllm.config import (
     VLLMChatConfig,
     VLLMEmbeddingConfig,
 )
-from draive.vllm.types import VLLMException
 
 __all__ = (
     "VLLM",
     "VLLMChatConfig",
     "VLLMEmbeddingConfig",
-    "VLLMException",
 )
