@@ -13,17 +13,20 @@ class VLLMAPI:
         "_client",
         "_default_headers",
         "_extra",
+        "_timeout",
     )
 
     def __init__(
         self,
         base_url: str | None = None,
         default_headers: Mapping[str, str] | None = None,
+        timeout: float | None = None,
         **extra: Any,
     ) -> None:
         self._base_url: str | None = base_url or getenv_str("VLLM_BASE_URL")
         self._default_headers: Mapping[str, str] | None = default_headers
         self._extra: Mapping[str, Any] = extra
+        self._timeout: float | None = timeout
         self._client: AsyncOpenAI = self._prepare_client()
 
     def _prepare_client(self) -> AsyncOpenAI:
@@ -33,6 +36,7 @@ class VLLMAPI:
             project="vllm",
             base_url=self._base_url,
             default_headers=self._default_headers,
+            timeout=self._timeout,
             **self._extra,
         )
 
